# Copyright 2016 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import netaddr

from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions

from neutron_tempest_plugin.api import base

from neutron_lib import constants as lib_constants

# 3 IP addresses are taken from every total for IPv4 these are reserved
DEFAULT_IP4_RESERVED = 3
# 2 IP addresses are taken from every total for IPv6 these are reserved
# I assume the reason for having one less than IPv4 is it does not have
# broadcast address
DEFAULT_IP6_RESERVED = 2

DELETE_TIMEOUT = 10
DELETE_SLEEP = 2


class NetworksIpAvailabilityTest(base.BaseAdminNetworkTest):
    """Tests Networks IP Availability

    Tests the following operations in the Neutron API using the REST client for
    Neutron:

        test total and used ips for net create
        test total and ips for net after subnet create
        test total and used ips for net after subnet and port create

    """

    @classmethod
    @utils.requires_ext(extension="network-ip-availability", service="network")
    def skip_checks(cls):
        super(NetworksIpAvailabilityTest, cls).skip_checks()

    @staticmethod
    def _get_availability(network, net_availability):
        if 'network_ip_availabilities' in net_availability:
            for availability in net_availability['network_ip_availabilities']:
                if availability['network_id'] == network['id']:
                    return availability
            raise exceptions.TempestException('Network IP Availability not '
                                              'found')
        else:
            return net_availability['network_ip_availability']

    def _get_used_ips(self, network, net_availability):
        availability = self._get_availability(network, net_availability)
        return availability and availability['used_ips']

    def _assert_total_and_used_ips(self, expected_used, expected_total,
                                   network, net_availability):
        availability = self._get_availability(network, net_availability)
        self.assertEqual(expected_total, availability['total_ips'])
        self.assertEqual(expected_used, availability['used_ips'])


def calc_total_ips(prefix, ip_version):
    # will calculate total ips after removing reserved.
    if ip_version == lib_constants.IP_VERSION_4:
        total_ips = 2 ** (lib_constants.IPv4_BITS -
                          prefix) - DEFAULT_IP4_RESERVED
    elif ip_version == lib_constants.IP_VERSION_6:
        total_ips = 2 ** (lib_constants.IPv6_BITS -
                          prefix) - DEFAULT_IP6_RESERVED
    return total_ips


class NetworksIpAvailabilityIPv4Test(NetworksIpAvailabilityTest):

    def setUp(self):
        super(NetworksIpAvailabilityIPv4Test, self).setUp()
        net_name = data_utils.rand_name('network')
        self.network = self.create_network(network_name=net_name)

    @decorators.idempotent_id('0f33cc8c-1bf6-47d1-9ce1-010618240599')
    def test_list_ip_availability_before_subnet(self):
        net_availability = self.admin_client.list_network_ip_availabilities()
        self._assert_total_and_used_ips(0, 0, self.network, net_availability)

    @decorators.idempotent_id('3aecd3b2-16ed-4b87-a54a-91d7b3c2986b')
    def test_list_ip_availability_after_subnet_and_ports(self):
        subnet = self.create_subnet(self.network, enable_dhcp=False)
        prefix = netaddr.IPNetwork(subnet['cidr']).prefixlen
        body = self.admin_client.list_network_ip_availabilities()
        used_ips_before_port_create = self._get_used_ips(self.network, body)
        self.create_port(self.network)
        net_availability = self.admin_client.list_network_ip_availabilities()
        self._assert_total_and_used_ips(
            used_ips_before_port_create + 1,
            calc_total_ips(prefix, self._ip_version),
            self.network, net_availability)

    @decorators.idempotent_id('9f11254d-757b-492e-b14b-f52144e4ee7b')
    def test_list_ip_availability_after_port_delete(self):
        self.create_subnet(self.network, enable_dhcp=False)
        port = self.create_port(self.network)
        net_availability = self.admin_client.list_network_ip_availabilities()
        used_ips = self._get_used_ips(self.network, net_availability)
        self.client.delete_port(port['id'])

        def is_count_ip_availability_valid():
            availabilities = self.admin_client.list_network_ip_availabilities()
            used_ips_after_port_delete = self._get_used_ips(self.network,
                                                            availabilities)
            return used_ips - 1 == used_ips_after_port_delete

        self.assertTrue(
            test_utils.call_until_true(
                is_count_ip_availability_valid, DELETE_TIMEOUT, DELETE_SLEEP),
            msg="IP address did not become available after port delete")

    @decorators.idempotent_id('da1fbed5-b4a9-45b3-bdcb-b1660710d565')
    def test_show_ip_availability_after_subnet_and_ports_create(self):
        net_availability = self.admin_client.show_network_ip_availability(
            self.network['id'])
        self._assert_total_and_used_ips(0, 0, self.network, net_availability)
        subnet = self.create_subnet(self.network, enable_dhcp=False)
        prefix = netaddr.IPNetwork(subnet['cidr']).prefixlen
        net_availability = self.admin_client.show_network_ip_availability(
            self.network['id'])
        used_ips_before_port_create = self._get_used_ips(self.network,
                                                         net_availability)
        self.create_port(self.network)
        net_availability = self.admin_client.show_network_ip_availability(
            self.network['id'])
        self._assert_total_and_used_ips(
            used_ips_before_port_create + 1,
            calc_total_ips(prefix, self._ip_version),
            self.network,
            net_availability)

    @decorators.idempotent_id('a4d1e291-c152-4d62-9316-8c9bf1c6aee2')
    def test_show_ip_availability_after_port_delete(self):
        self.create_subnet(self.network, enable_dhcp=False)
        port = self.create_port(self.network)
        net_availability = self.admin_client.show_network_ip_availability(
            self.network['id'])
        used_ips = self._get_used_ips(self.network, net_availability)
        self.client.delete_port(port['id'])

        def is_count_ip_availability_valid():
            availabilities = self.admin_client.show_network_ip_availability(
                self.network['id'])
            used_ips_after_port_delete = self._get_used_ips(self.network,
                                                            availabilities)
            return used_ips - 1 == used_ips_after_port_delete

        self.assertTrue(
            test_utils.call_until_true(
                is_count_ip_availability_valid, DELETE_TIMEOUT, DELETE_SLEEP),
            msg="IP address did not become available after port delete")


class NetworksIpAvailabilityIPv6Test(NetworksIpAvailabilityIPv4Test):

    _ip_version = lib_constants.IP_VERSION_6

    @decorators.idempotent_id('0d5a03f2-fdb7-4ec3-b746-734c51d74b69')
    def test_list_ipv6_ip_availability_after_subnet_and_ports(self):
        subnet = self.create_subnet(self.network, ip_version=self._ip_version,
                                    enable_dhcp=False)
        prefix = netaddr.IPNetwork(subnet['cidr']).prefixlen
        body = self.admin_client.list_network_ip_availabilities()
        used_ips_before_port_create = self._get_used_ips(self.network, body)
        self.create_port(self.network)
        net_availability = self.admin_client.list_network_ip_availabilities()
        self._assert_total_and_used_ips(
            used_ips_before_port_create + 1,
            calc_total_ips(prefix, self._ip_version),
            self.network, net_availability)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from neutron_lib import constants
from tempest.lib import decorators
import testtools

from neutron_tempest_plugin.api import base
from neutron_tempest_plugin import config

AZ_SUPPORTED_AGENTS = [constants.AGENT_TYPE_DHCP, constants.AGENT_TYPE_L3]
CONF = config.CONF


class AgentAvailabilityZoneTestCase(base.BaseAdminNetworkTest):

    required_extensions = ['agent', 'availability_zone']

    @classmethod
    def resource_setup(cls):
        super(AgentAvailabilityZoneTestCase, cls).resource_setup()
        body = cls.admin_client.list_agents()
        agents = body['agents']
        agents_type = [agent.get('agent_type') for agent in agents]
        for az_agent in AZ_SUPPORTED_AGENTS:
            if az_agent in agents_type:
                return
        msg = 'availability_zone supported agent not found.'
        raise cls.skipException(msg)

    @decorators.idempotent_id('3ffa661e-cfcc-417d-8b63-1c5ec4a22e54')
    @testtools.skipUnless(CONF.neutron_plugin_options.agent_availability_zone,
                          "Need a single availability_zone assumption.")
    def test_agents_availability_zone(self):
        """Test list agents availability_zone

        Only L3 and DHCP agent support availability_zone, default
        availability_zone is "nova".
        """
        body = self.admin_client.list_agents()
        agents = body['agents']
        for agent in agents:
            if agent.get('agent_type') in AZ_SUPPORTED_AGENTS:
                self.assertEqual(
                    CONF.neutron_plugin_options.agent_availability_zone,
                    agent.get('availability_zone'))



# Copyright (c) 2012 OpenStack Foundation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from neutron_lib import constants as lib_constants


ROUTER_PORT_OWNERS = lib_constants.ROUTER_INTERFACE_OWNERS_SNAT + \
    (lib_constants.DEVICE_OWNER_ROUTER_GW,)

ROUTER_STATUS_ACTIVE = 'ACTIVE'
ROUTER_STATUS_ERROR = 'ERROR'

DEVICE_ID_RESERVED_DHCP_PORT = "reserved_dhcp_port"

HA_ROUTER_STATE_KEY = '_ha_state'
METERING_LABEL_KEY = '_metering_labels'
FLOATINGIP_AGENT_INTF_KEY = '_floatingip_agent_interfaces'
SNAT_ROUTER_INTF_KEY = '_snat_router_interfaces'
DVR_SNAT_BOUND = 'dvr_snat_bound'
L3_AGENT_MODE_DVR_NO_EXTERNAL = 'dvr_no_external'

HA_NETWORK_NAME = 'HA network tenant %s'
HA_SUBNET_NAME = 'HA subnet tenant %s'
HA_PORT_NAME = 'HA port tenant %s'
HA_ROUTER_STATE_ACTIVE = 'active'
HA_ROUTER_STATE_STANDBY = 'standby'

PAGINATION_INFINITE = 'infinite'

SORT_DIRECTION_ASC = 'asc'
SORT_DIRECTION_DESC = 'desc'

ETHERTYPE_NAME_ARP = 'arp'
ETHERTYPE_ARP = 0x0806
ETHERTYPE_IP = 0x0800
ETHERTYPE_IPV6 = 0x86DD

IP_PROTOCOL_NAME_ALIASES = {lib_constants.PROTO_NAME_IPV6_ICMP_LEGACY:
                            lib_constants.PROTO_NAME_IPV6_ICMP}

VALID_DSCP_MARKS = [0, 8, 10, 12, 14, 16, 18, 20, 22, 24, 26, 28, 30, 32, 34,
                    36, 38, 40, 46, 48, 56]

IP_PROTOCOL_NUM_TO_NAME_MAP = {
    str(v): k for k, v in lib_constants.IP_PROTOCOL_MAP.items()}

# When using iptables-save we specify '-p {proto} -m {module}',
# but sometimes those values are not identical.  This is a map
# of known protocols that require a '-m {module}', along with
# the module name that should be used.
IPTABLES_PROTOCOL_MAP = {lib_constants.PROTO_NAME_DCCP: 'dccp',
                         lib_constants.PROTO_NAME_ICMP: 'icmp',
                         lib_constants.PROTO_NAME_IPV6_ICMP: 'icmp6',
                         lib_constants.PROTO_NAME_SCTP: 'sctp',
                         lib_constants.PROTO_NAME_TCP: 'tcp',
                         lib_constants.PROTO_NAME_UDP: 'udp'}

# Special provisional prefix for IPv6 Prefix Delegation
PROVISIONAL_IPV6_PD_PREFIX = '::/64'

# Timeout in seconds for getting an IPv6 LLA
LLA_TASK_TIMEOUT = 40

# length of all device prefixes (e.g. qvo, tap, qvb)
LINUX_DEV_PREFIX_LEN = 3
# must be shorter than linux IFNAMSIZ (which is 16)
LINUX_DEV_LEN = 14

# Possible prefixes to partial port IDs in interface names used by the OVS,
# Linux Bridge, and IVS VIF drivers in Nova and the neutron agents. See the
# 'get_ovs_interfaceid' method in Nova (nova/virt/libvirt/vif.py) for details.
INTERFACE_PREFIXES = (lib_constants.TAP_DEVICE_PREFIX,
                      lib_constants.VETH_DEVICE_PREFIX,
                      lib_constants.SNAT_INT_DEV_PREFIX)

ATTRIBUTES_TO_UPDATE = 'attributes_to_update'

# Maximum value integer can take in MySQL and PostgreSQL
# In SQLite integer can be stored in 1, 2, 3, 4, 6, or 8 bytes,
# but here it will be limited by this value for consistency.
DB_INTEGER_MAX_VALUE = 2 ** 31 - 1

# TODO(amuller): Re-define the RPC namespaces once Oslo messaging supports
# Targets with multiple namespaces. Neutron will then implement callbacks
# for its RPC clients in order to support rolling upgrades.

# RPC Interface for agents to call DHCP API implemented on the plugin side
RPC_NAMESPACE_DHCP_PLUGIN = None
# RPC interface for the metadata service to get info from the plugin side
RPC_NAMESPACE_METADATA = None
# RPC interface for agent to plugin security group API
RPC_NAMESPACE_SECGROUP = None
# RPC interface for agent to plugin DVR api
RPC_NAMESPACE_DVR = None
# RPC interface for reporting state back to the plugin
RPC_NAMESPACE_STATE = None
# RPC interface for agent to plugin resources API
RPC_NAMESPACE_RESOURCES = None

# Default network MTU value when not configured
DEFAULT_NETWORK_MTU = 1500
IPV6_MIN_MTU = 1280

ROUTER_MARK_MASK = "0xffff"

# Agent states as detected by server, used to reply on agent's state report
# agent has just been registered
AGENT_NEW = 'new'
# agent is alive
AGENT_ALIVE = 'alive'
# agent has just returned to alive after being dead
AGENT_REVIVED = 'revived'

VALID_ETHERTYPES = (lib_constants.IPv4, lib_constants.IPv6)

IP_ALLOWED_VERSIONS = [lib_constants.IP_VERSION_4, lib_constants.IP_VERSION_6]

PORT_RANGE_MIN = 1
PORT_RANGE_MAX = 65535

# Configuration values for accept_ra sysctl, copied from linux kernel
# networking (netdev) tree, file Documentation/networking/ip-sysctl.txt
#
# Possible values are:
#         0 Do not accept Router Advertisements.
#         1 Accept Router Advertisements if forwarding is disabled.
#         2 Overrule forwarding behaviour. Accept Router Advertisements
#           even if forwarding is enabled.
ACCEPT_RA_DISABLED = 0
ACCEPT_RA_WITHOUT_FORWARDING = 1
ACCEPT_RA_WITH_FORWARDING = 2

# Some components communicate using private address ranges, define
# them all here. These address ranges should not cause any issues
# even if they overlap since they are used in disjoint namespaces,
# but for now they are unique.
# We define the metadata cidr since it falls in the range.
PRIVATE_CIDR_RANGE = '169.254.0.0/16'
DVR_FIP_LL_CIDR = '169.254.64.0/18'
L3_HA_NET_CIDR = '169.254.192.0/18'
METADATA_CIDR = '169.254.169.254/32'

# The only defined IpamAllocation status at this stage is 'ALLOCATED'.
# More states will be available in the future - e.g.: RECYCLABLE
IPAM_ALLOCATION_STATUS_ALLOCATED = 'ALLOCATED'

VALID_IPAM_ALLOCATION_STATUSES = (IPAM_ALLOCATION_STATUS_ALLOCATED,)

# Port binding states for Live Migration
PORT_BINDING_STATUS_ACTIVE = 'ACTIVE'
PORT_BINDING_STATUS_INACTIVE = 'INACTIVE'
PORT_BINDING_STATUSES = (PORT_BINDING_STATUS_ACTIVE,
                         PORT_BINDING_STATUS_INACTIVE)

VALID_FLOATINGIP_STATUS = (lib_constants.FLOATINGIP_STATUS_ACTIVE,
                           lib_constants.FLOATINGIP_STATUS_DOWN,
                           lib_constants.FLOATINGIP_STATUS_ERROR)

# Possible types of values (e.g. in QoS rule types)
VALUES_TYPE_CHOICES = "choices"
VALUES_TYPE_RANGE = "range"

# Security group parameters values mapped by IP version
DEFAULT_SECURITY_GROUP_RULE_PARAMS = {
    lib_constants.IP_VERSION_4: {'ethertype': lib_constants.IPv4,
                                 'remote_ip_prefix': lib_constants.IPv4_ANY},
    lib_constants.IP_VERSION_6: {'ethertype': lib_constants.IPv6,
                                 'remote_ip_prefix': lib_constants.IPv6_ANY},
}



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib import constants
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
import testtools

from neutron_tempest_plugin.api import base_routers as base
from neutron_tempest_plugin import config


CONF = config.CONF


class RoutersNegativeTestBase(base.BaseRouterTest):

    required_extensions = ['router']

    @classmethod
    def resource_setup(cls):
        super(RoutersNegativeTestBase, cls).resource_setup()
        cls.router = cls.create_router(data_utils.rand_name('router'))
        cls.network = cls.create_network()
        cls.subnet = cls.create_subnet(cls.network)


class RoutersNegativePolicyTest(RoutersNegativeTestBase):

    credentials = ['admin', 'primary', 'alt']

    @decorators.attr(type='negative')
    @decorators.idempotent_id('159f576d-a423-46b5-b501-622694c02f6b')
    def test_add_interface_wrong_tenant(self):
        client2 = self.os_alt.network_client
        network = client2.create_network()['network']
        self.addCleanup(client2.delete_network, network['id'])
        subnet = self.create_subnet(network, client=client2)
        # This port is deleted after a test by remove_router_interface.
        port = client2.create_port(network_id=network['id'])['port']
        self.addCleanup(client2.delete_port, port['id'])
        with testtools.ExpectedException(lib_exc.NotFound):
            client2.add_router_interface_with_port_id(
                self.router['id'], port['id'])
        with testtools.ExpectedException(lib_exc.NotFound):
            client2.add_router_interface_with_subnet_id(
                self.router['id'], subnet['id'])

    @decorators.attr(type='negative')
    @decorators.idempotent_id('8010d27e-4ab7-4ea3-98b1-3995b7910efd')
    def test_add_interface_in_use(self):
        port = self.create_port(self.network)
        self.client.add_router_interface_with_port_id(
            self.router['id'], port['id'])
        self.assertRaises(
            lib_exc.Conflict,
            self.client.add_router_interface_with_port_id,
            self.router['id'], port['id'])

    @decorators.attr(type='negative')
    @decorators.idempotent_id('ed84c800-ee29-4b76-9419-d6d7b143fc47')
    def test_add_interface_port_nonexist(self):
        # port id is not a valid UUID
        invalid_id = data_utils.rand_name('port')
        self.assertRaises(
            lib_exc.BadRequest,
            self.client.add_router_interface_with_port_id,
            self.router['id'], invalid_id)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('dad7a8ba-2726-11eb-82dd-74e5f9e2a801')
    def test_remove_associated_ports(self):
        self.client.update_router(
            self.router['id'],
            external_gateway_info={
                'network_id': CONF.network.public_network_id})
        network = self.create_network()
        subnet = self.create_subnet(network)
        self.create_router_interface(self.router['id'], subnet['id'])
        port_ids = [
            item['id'] for item in self.admin_client.list_ports(
                device_id=self.router['id'])['ports']
            if item['device_owner'] not in [
                constants.DEVICE_OWNER_ROUTER_HA_INTF,
                constants.DEVICE_OWNER_HA_REPLICATED_INT]]
        if not port_ids:
            self.fail("No ports other than HA ports found for the router %s" %
                      self.router['id'])
        for port_id in port_ids:
            with testtools.ExpectedException(lib_exc.Conflict):
                self.admin_client.delete_port(port_id)


class DvrRoutersNegativeTest(RoutersNegativeTestBase):

    required_extensions = ['dvr']

    @decorators.attr(type='negative')
    @decorators.idempotent_id('4990b055-8fc7-48ab-bba7-aa28beaad0b9')
    def test_router_create_tenant_distributed_returns_forbidden(self):
        with testtools.ExpectedException(lib_exc.Forbidden):
            self.create_router(
                data_utils.rand_name('router'), distributed=True)


class DvrRoutersNegativeTestExtended(RoutersNegativeTestBase):

    required_extensions = ['dvr', 'router-admin-state-down-before-update']

    @decorators.attr(type='negative')
    @decorators.idempotent_id('5379fe06-e45e-4a4f-8b4a-9e28a924b451')
    def test_router_update_distributed_returns_exception(self):
        # create a centralized router
        router_args = {'tenant_id': self.client.project_id,
                       'distributed': False}
        router = self._create_admin_router(
            data_utils.rand_name('router'), admin_state_up=True,
            **router_args)
        self.assertTrue(router['admin_state_up'])
        self.assertFalse(router['distributed'])
        # attempt to set the router to distributed, catch BadRequest exception
        self.assertRaises(lib_exc.BadRequest,
                          self.admin_client.update_router,
                          router['id'],
                          distributed=True)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('c277e945-3b39-442d-b149-e2e8cc6a2b40')
    def test_router_update_centralized_returns_exception(self):
        # create a centralized router
        router_args = {'tenant_id': self.client.project_id,
                       'distributed': False}
        router = self._create_admin_router(
            data_utils.rand_name('router'), admin_state_up=True,
            **router_args)
        self.assertTrue(router['admin_state_up'])
        self.assertFalse(router['distributed'])
        # take the router down to modify distributed->True
        update_body = self.admin_client.update_router(router['id'],
                                                      admin_state_up=False)
        self.assertFalse(update_body['router']['admin_state_up'])
        update_body = self.admin_client.update_router(router['id'],
                                                      distributed=True)
        self.assertTrue(update_body['router']['distributed'])
        # set admin_state_up=True
        update_body = self.admin_client.update_router(router['id'],
                                                      admin_state_up=True)
        self.assertTrue(update_body['router']['admin_state_up'])
        # attempt to set the router to centralized, catch BadRequest exception
        self.assertRaises(lib_exc.BadRequest,
                          self.admin_client.update_router,
                          router['id'],
                          distributed=False)


class HaRoutersNegativeTest(RoutersNegativeTestBase):

    required_extensions = ['l3-ha']

    @decorators.attr(type='negative')
    @decorators.idempotent_id('821b85b9-9c51-40f3-831f-bf223a7e0084')
    def test_router_create_tenant_ha_returns_forbidden(self):
        with testtools.ExpectedException(lib_exc.Forbidden):
            self.create_router(
                data_utils.rand_name('router'), ha=True)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
import time

from neutron_lib import constants
from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from neutron_tempest_plugin.api import base
from neutron_tempest_plugin.api import base_routers
from neutron_tempest_plugin import config

CONF = config.CONF


class TestTimeStamp(base.BaseAdminNetworkTest):

    required_extensions = ["standard-attr-timestamp"]

    # attributes for subnetpool
    min_prefixlen = '28'
    max_prefixlen = '31'
    _ip_version = 4
    subnet_cidr = '10.11.12.0/31'
    new_prefix = '10.11.15.0/24'
    larger_prefix = '10.11.0.0/16'

    @classmethod
    def resource_setup(cls):
        super(TestTimeStamp, cls).resource_setup()
        prefixes = ['10.11.12.0/24']
        cls._subnetpool_data = {'min_prefixlen': '29', 'prefixes': prefixes}

    def _create_subnetpool(self, is_admin=False, **kwargs):
        name = data_utils.rand_name('subnetpool')
        subnetpool_data = copy.deepcopy(self._subnetpool_data)
        for key in subnetpool_data.keys():
            kwargs[key] = subnetpool_data[key]
        return self.create_subnetpool(name=name, is_admin=is_admin, **kwargs)

    @decorators.idempotent_id('462be770-b310-4df9-9c42-773217e4c8b1')
    def test_create_network_with_timestamp(self):
        network = self.create_network()
        # Verifies body contains timestamp fields
        self.assertIsNotNone(network['created_at'])
        self.assertIsNotNone(network['updated_at'])

    @decorators.idempotent_id('4db5417a-e11c-474d-a361-af00ebef57c5')
    def test_update_network_with_timestamp(self):
        network = self.create_network()
        origin_updated_at = network['updated_at']
        update_body = {'name': network['name'] + 'new'}
        body = self.admin_client.update_network(network['id'],
                                                **update_body)
        updated_network = body['network']
        new_updated_at = updated_network['updated_at']
        self.assertEqual(network['created_at'], updated_network['created_at'])
        # Verify that origin_updated_at is not same with new_updated_at
        self.assertIsNot(origin_updated_at, new_updated_at)

    @decorators.idempotent_id('2ac50ab2-7ebd-4e27-b3ce-a9e399faaea2')
    def test_show_networks_attribute_with_timestamp(self):
        network = self.create_network()
        body = self.client.show_network(network['id'])
        show_net = body['network']
        # verify the timestamp from creation and showed is same
        self.assertEqual(network['created_at'],
                         show_net['created_at'])
        self.assertEqual(network['updated_at'],
                         show_net['updated_at'])

    @decorators.idempotent_id('8ee55186-454f-4b97-9f9f-eb2772ee891c')
    def test_create_subnet_with_timestamp(self):
        network = self.create_network()
        subnet = self.create_subnet(network)
        # Verifies body contains timestamp fields
        self.assertIsNotNone(subnet['created_at'])
        self.assertIsNotNone(subnet['updated_at'])

    @decorators.idempotent_id('a490215a-6f4c-4af9-9a4c-57c41f1c4fa1')
    def test_update_subnet_with_timestamp(self):
        network = self.create_network()
        subnet = self.create_subnet(network)
        origin_updated_at = subnet['updated_at']
        update_body = {'name': subnet['name'] + 'new'}
        body = self.admin_client.update_subnet(subnet['id'],
                                               **update_body)
        updated_subnet = body['subnet']
        new_updated_at = updated_subnet['updated_at']
        self.assertEqual(subnet['created_at'], updated_subnet['created_at'])
        # Verify that origin_updated_at is not same with new_updated_at
        self.assertIsNot(origin_updated_at, new_updated_at)

    @decorators.idempotent_id('1836a086-e7cf-4141-bf57-0cfe79e8051e')
    def test_show_subnet_attribute_with_timestamp(self):
        network = self.create_network()
        subnet = self.create_subnet(network)
        body = self.client.show_subnet(subnet['id'])
        show_subnet = body['subnet']
        # verify the timestamp from creation and showed is same
        self.assertEqual(subnet['created_at'],
                         show_subnet['created_at'])
        self.assertEqual(subnet['updated_at'],
                         show_subnet['updated_at'])

    @decorators.idempotent_id('e2450a7b-d84f-4600-a093-45e78597bbac')
    def test_create_port_with_timestamp(self):
        network = self.create_network()
        port = self.create_port(network)
        # Verifies body contains timestamp fields
        self.assertIsNotNone(port['created_at'])
        self.assertIsNotNone(port['updated_at'])

    @decorators.idempotent_id('4241e0d3-54b4-46ce-a9a7-093fc764161b')
    def test_update_port_with_timestamp(self):
        network = self.create_network()
        port = self.create_port(network)
        origin_updated_at = port['updated_at']
        update_body = {'name': port['name'] + 'new'}
        body = self.admin_client.update_port(port['id'],
                                             **update_body)
        updated_port = body['port']
        new_updated_at = updated_port['updated_at']
        self.assertEqual(port['created_at'], updated_port['created_at'])
        # Verify that origin_updated_at is not same with new_updated_at
        self.assertIsNot(origin_updated_at, new_updated_at)

    @decorators.idempotent_id('584c6723-40b6-4f26-81dd-f508f9d9fb51')
    def test_show_port_attribute_with_timestamp(self):
        network = self.create_network()
        port = self.create_port(network)
        body = self.client.show_port(port['id'])
        show_port = body['port']
        # verify the timestamp from creation and showed is same
        self.assertEqual(port['created_at'],
                         show_port['created_at'])
        self.assertEqual(port['updated_at'],
                         show_port['updated_at'])

    @decorators.idempotent_id('87a8b196-4b90-44f0-b7f3-d2057d7d658e')
    def test_create_subnetpool_with_timestamp(self):
        sp = self._create_subnetpool()
        # Verifies body contains timestamp fields
        self.assertIsNotNone(sp['created_at'])
        self.assertIsNotNone(sp['updated_at'])

    @decorators.idempotent_id('d48c7578-c3d2-4f9b-a7a1-be2008c770a0')
    def test_update_subnetpool_with_timestamp(self):
        sp = self._create_subnetpool()
        origin_updated_at = sp['updated_at']
        update_body = {'name': sp['name'] + 'new',
                       'min_prefixlen': self.min_prefixlen,
                       'max_prefixlen': self.max_prefixlen}
        body = self.client.update_subnetpool(sp['id'], **update_body)
        updated_sp = body['subnetpool']
        new_updated_at = updated_sp['updated_at']
        self.assertEqual(sp['created_at'], updated_sp['created_at'])
        # Verify that origin_updated_at is not same with new_updated_at
        self.assertIsNot(origin_updated_at, new_updated_at)

    @decorators.idempotent_id('1d3970e6-bcf7-46cd-b7d7-0807759c73b4')
    def test_show_subnetpool_attribute_with_timestamp(self):
        sp = self._create_subnetpool()
        body = self.client.show_subnetpool(sp['id'])
        show_sp = body['subnetpool']
        # verify the timestamp from creation and showed is same
        self.assertEqual(sp['created_at'], show_sp['created_at'])
        self.assertEqual(sp['updated_at'], show_sp['updated_at'])

    @decorators.idempotent_id('396a97dc-b66c-4c46-9171-c39eefe6936c')
    @utils.requires_ext(extension="standard-attr-segment", service="network")
    def test_segment_with_timestamp(self):
        network = self.create_network()
        segment = self.admin_client.list_segments(
            network_id=network['id'])['segments'][0]
        # Verifies body contains timestamp fields
        self.assertIsNotNone(segment['created_at'])
        self.assertIsNotNone(segment['updated_at'])

        body = self.admin_client.show_segment(segment['id'])
        show_segment = body['segment']
        # verify the timestamp from creation and showed is same
        self.assertEqual(segment['created_at'], show_segment['created_at'])
        self.assertEqual(segment['updated_at'], show_segment['updated_at'])

        origin_updated_at = segment['updated_at']
        update_body = {'name': str(segment['name']) + 'new'}
        body = self.admin_client.update_segment(segment['id'], **update_body)
        updated_segment = body['segment']
        new_updated_at = updated_segment['updated_at']
        self.assertEqual(segment['created_at'], updated_segment['created_at'])
        # Verify that origin_updated_at is not same with new_updated_at
        self.assertIsNot(origin_updated_at, new_updated_at)


class TestTimeStampWithL3(base_routers.BaseRouterTest):

    required_extensions = ['standard-attr-timestamp']

    @classmethod
    def resource_setup(cls):
        super(TestTimeStampWithL3, cls).resource_setup()
        cls.ext_net_id = CONF.network.public_network_id

    @decorators.idempotent_id('433ba770-b310-4da9-5d42-733217a1c7b1')
    def test_create_router_with_timestamp(self):
        router = self.create_router(router_name='test')
        # Verifies body contains timestamp fields
        self.assertIsNotNone(router['created_at'])
        self.assertIsNotNone(router['updated_at'])

    @decorators.idempotent_id('4a65417a-c11c-4b4d-a351-af01abcf57c6')
    def test_update_router_with_timestamp(self):
        router = self.create_router(router_name='test')
        origin_updated_at = router['updated_at']
        update_body = {'name': router['name'] + 'new'}
        body = self.client.update_router(router['id'], **update_body)
        updated_router = body['router']
        new_updated_at = updated_router['updated_at']
        self.assertEqual(router['created_at'], updated_router['created_at'])
        # Verify that origin_updated_at is not same with new_updated_at
        self.assertIsNot(origin_updated_at, new_updated_at)

    @decorators.idempotent_id('1ab50ac2-7cbd-4a17-b23e-a9e36cfa4ec2')
    def test_show_router_attribute_with_timestamp(self):
        router = self.create_router(router_name='test')
        body = self.client.show_router(router['id'])
        show_router = body['router']
        # verify the timestamp from creation and showed is same
        self.assertEqual(router['created_at'],
                         show_router['created_at'])
        # 'updated_at' timestamp can change immediately after creation
        # if environment is HA or DVR, so just make sure it's >=
        self.assertGreaterEqual(show_router['updated_at'],
                                router['updated_at'])

    @decorators.idempotent_id('8ae55186-464f-4b87-1c9f-eb2765ee81ac')
    def test_create_floatingip_with_timestamp(self):
        fip = self.create_floatingip()
        # Verifies body contains timestamp fields
        self.assertIsNotNone(fip['created_at'])
        self.assertIsNotNone(fip['updated_at'])

    @decorators.idempotent_id('a3ac215a-61ac-13f9-9d3c-57c51f11afa1')
    def test_update_floatingip_with_timestamp(self):
        fip = self.create_floatingip()
        origin_updated_at = fip['updated_at']
        update_body = {'description': 'new'}
        body = self.client.update_floatingip(fip['id'], **update_body)
        updated_fip = body['floatingip']
        new_updated_at = updated_fip['updated_at']
        self.assertEqual(fip['created_at'], updated_fip['created_at'])
        # Verify that origin_updated_at is not same with new_updated_at
        self.assertIsNot(origin_updated_at, new_updated_at)

    @decorators.idempotent_id('32a6a086-e1ef-413b-b13a-0cfe13ef051e')
    def test_show_floatingip_attribute_with_timestamp(self):
        fip = self.create_floatingip()
        body = self.client.show_floatingip(fip['id'])
        show_fip = body['floatingip']
        # verify the timestamp from creation and showed is same
        self.assertEqual(fip['created_at'],
                         show_fip['created_at'])
        self.assertEqual(fip['updated_at'],
                         show_fip['updated_at'])


class TestTimeStampWithSecurityGroup(base.BaseNetworkTest):

    required_extensions = ['standard-attr-timestamp']

    @classmethod
    def resource_setup(cls):
        super(TestTimeStampWithSecurityGroup, cls).resource_setup()
        cls.ext_net_id = CONF.network.public_network_id

    @decorators.idempotent_id('a3150a7b-d31a-423a-abf3-45e71c97cbac')
    def test_create_sg_with_timestamp(self):
        security_group = self.create_security_group()
        # Verifies body contains timestamp fields
        self.assertIsNotNone(security_group['created_at'])
        self.assertIsNotNone(security_group['updated_at'])

    @decorators.idempotent_id('432ae0d3-32b4-413e-a9b3-091ac76da31b')
    def test_update_sg_with_timestamp(self):
        security_group = self.create_security_group()

        # Make sure update time will be different
        time.sleep(2.)
        updated_security_group = self.client.update_security_group(
            security_group['id'], name=security_group['name'] + 'new')[
                'security_group']

        # Verify that created_at hasn't changed
        self.assertEqual(security_group['created_at'],
                         updated_security_group['created_at'])
        # Verify that updated_at has changed
        self.assertNotEqual(security_group['updated_at'],
                            updated_security_group['updated_at'])

    @decorators.idempotent_id('521e6723-43d6-12a6-8c3d-f5042ad9fc32')
    def test_show_sg_attribute_with_timestamp(self):
        security_group = self.create_security_group()
        observed_security_group = self.client.show_security_group(
            security_group['id'])['security_group']

        # Verify that created_at hasn't changed
        self.assertEqual(security_group['created_at'],
                         observed_security_group['created_at'])
        # Verify that updated_at hasn't changed
        self.assertEqual(security_group['updated_at'],
                         observed_security_group['updated_at'])

    def _create_security_group_rule(self):
        security_group = self.create_security_group()
        return self.create_security_group_rule(
            security_group=security_group,
            direction=constants.INGRESS_DIRECTION,
            protocol=constants.PROTO_NAME_TCP,
            port_range_min=77,
            port_range_max=77)

    @decorators.idempotent_id('83e8bd32-43e0-a3f0-1af3-12a5733c653e')
    def test_create_sgrule_with_timestamp(self):
        security_group_rule = self._create_security_group_rule()
        # Verifies body contains timestamp fields
        self.assertIn('created_at', security_group_rule)
        self.assertIn('updated_at', security_group_rule)

    @decorators.idempotent_id('143da0e6-ba17-43ad-b3d7-03aa759c3cb4')
    def test_show_sgrule_attribute_with_timestamp(self):
        security_group_rule = self._create_security_group_rule()

        observed_security_group_rule = self.client.show_security_group_rule(
            security_group_rule['id'])['security_group_rule']

        # Verify the time stamp from creation and showed are equal
        self.assertEqual(security_group_rule['created_at'],
                         observed_security_group_rule['created_at'])
        self.assertEqual(security_group_rule['updated_at'],
                         observed_security_group_rule['updated_at'])



# Copyright 2016 Futurewei. All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from neutron_tempest_plugin.sfc.tests.api import base


class FlowClassifierExtensionTestJSON(base.BaseFlowClassifierTest):
    """Tests the following operations in the Neutron API:

        List flowclassifiers
        Create flowclassifier
        Update flowclassifier
        Delete flowclassifier
        Show flowclassifier
    """

    @decorators.idempotent_id('1b84cf01-9c09-4ce7-bc72-b15e39076468')
    def test_list_flowclassifier(self):
        # List flow classifiers
        fc = self._try_create_flowclassifier()
        fcs = self.flowclassifier_client.list_flowclassifiers()
        self.assertIn((
            fc['id'],
            fc['name'],
            fc['source_ip_prefix'],
            fc['logical_source_port']
        ), [(
            m['id'],
            m['name'],
            m['source_ip_prefix'],
            m['logical_source_port'],
        ) for m in fcs['flow_classifiers']])

    @decorators.idempotent_id('b2ed2a37-fc64-4be5-819b-9cf2a13db70b')
    def test_list_flowclassifier_with_logical_destination_port(self):
        # List flow classifiers with logical_destination_port
        fc = self._try_create_flowclassifier()
        fcs = self.flowclassifier_client.list_flowclassifiers()
        self.assertIn((
            fc['id'],
            fc['name'],
            fc['source_ip_prefix'],
            fc['destination_ip_prefix'],
            fc['logical_source_port'],
            fc['logical_destination_port']
        ), [(
            m['id'],
            m['name'],
            m['source_ip_prefix'],
            m['destination_ip_prefix'],
            m['logical_source_port'],
            m['logical_destination_port']
        ) for m in fcs['flow_classifiers']])

    @decorators.idempotent_id('563564f7-7077-4f5e-8cdc-51f37ae5a2b9')
    def test_update_flowclassifier(self):
        # Create flow classifier
        name1 = data_utils.rand_name('test')
        fc = self._try_create_flowclassifier(
            name=name1
        )
        fc_id = fc['id']

        # Update flow classifier
        name2 = data_utils.rand_name('test')
        body = self.flowclassifier_client.update_flowclassifier(
            fc_id, name=name2)
        self.assertEqual(body['flow_classifier']['name'], name2)

    @decorators.idempotent_id('3ff8c08e-26ff-4034-ae48-810ed213a998')
    def test_show_flowclassifier(self):
        # show a created flow classifier
        created = self._try_create_flowclassifier()
        fc = self.flowclassifier_client.show_flowclassifier(
            created['id'])
        for key, value in fc['flow_classifier'].items():
            self.assertEqual(created[key], value)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import functools
import math
import time

import netaddr
from neutron_lib import constants as const
from oslo_log import log
from tempest.common import utils as tutils
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions as lib_exc
from tempest import test

from neutron_tempest_plugin.api import clients
from neutron_tempest_plugin.common import constants
from neutron_tempest_plugin.common import utils
from neutron_tempest_plugin import config
from neutron_tempest_plugin import exceptions

CONF = config.CONF

LOG = log.getLogger(__name__)


class BaseNetworkTest(test.BaseTestCase):

    """Base class for Neutron tests that use the Tempest Neutron REST client

    Per the Neutron API Guide, API v1.x was removed from the source code tree
    (docs.openstack.org/api/openstack-network/2.0/content/Overview-d1e71.html)
    Therefore, v2.x of the Neutron API is assumed. It is also assumed that the
    following options are defined in the [network] section of etc/tempest.conf:

        project_network_cidr with a block of cidr's from which smaller blocks
        can be allocated for tenant networks

        project_network_mask_bits with the mask bits to be used to partition
        the block defined by tenant-network_cidr

    Finally, it is assumed that the following option is defined in the
    [service_available] section of etc/tempest.conf

        neutron as True
    """

    force_tenant_isolation = False
    credentials = ['primary']

    # Default to ipv4.
    _ip_version = const.IP_VERSION_4

    # Derive from BaseAdminNetworkTest class to have this initialized
    admin_client = None

    external_network_id = CONF.network.public_network_id

    __is_driver_ovn = None

    @classmethod
    def _is_driver_ovn(cls):
        ovn_agents = cls.os_admin.network_client.list_agents(
            binary='ovn-controller')['agents']
        return len(ovn_agents) > 0

    @property
    def is_driver_ovn(self):
        if self.__is_driver_ovn is None:
            if hasattr(self, 'os_admin'):
                self.__is_driver_ovn = self._is_driver_ovn()
        return self.__is_driver_ovn

    @classmethod
    def get_client_manager(cls, credential_type=None, roles=None,
                           force_new=None):
        manager = super(BaseNetworkTest, cls).get_client_manager(
            credential_type=credential_type,
            roles=roles,
            force_new=force_new
        )
        # Neutron uses a different clients manager than the one in the Tempest
        # save the original in case mixed tests need it
        if credential_type == 'primary':
            cls.os_tempest = manager
        return clients.Manager(manager.credentials)

    @classmethod
    def skip_checks(cls):
        super(BaseNetworkTest, cls).skip_checks()
        if not CONF.service_available.neutron:
            raise cls.skipException("Neutron support is required")
        if (cls._ip_version == const.IP_VERSION_6 and
                not CONF.network_feature_enabled.ipv6):
            raise cls.skipException("IPv6 Tests are disabled.")
        for req_ext in getattr(cls, 'required_extensions', []):
            if not tutils.is_extension_enabled(req_ext, 'network'):
                msg = "%s extension not enabled." % req_ext
                raise cls.skipException(msg)

    @classmethod
    def setup_credentials(cls):
        # Create no network resources for these test.
        cls.set_network_resources()
        super(BaseNetworkTest, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(BaseNetworkTest, cls).setup_clients()
        cls.client = cls.os_primary.network_client

    @classmethod
    def resource_setup(cls):
        super(BaseNetworkTest, cls).resource_setup()

        cls.networks = []
        cls.admin_networks = []
        cls.subnets = []
        cls.admin_subnets = []
        cls.ports = []
        cls.routers = []
        cls.floating_ips = []
        cls.port_forwardings = []
        cls.local_ips = []
        cls.local_ip_associations = []
        cls.metering_labels = []
        cls.service_profiles = []
        cls.flavors = []
        cls.metering_label_rules = []
        cls.qos_rules = []
        cls.qos_policies = []
        cls.ethertype = "IPv" + str(cls._ip_version)
        cls.address_groups = []
        cls.admin_address_groups = []
        cls.address_scopes = []
        cls.admin_address_scopes = []
        cls.subnetpools = []
        cls.admin_subnetpools = []
        cls.security_groups = []
        cls.admin_security_groups = []
        cls.sg_rule_templates = []
        cls.projects = []
        cls.log_objects = []
        cls.reserved_subnet_cidrs = set()
        cls.keypairs = []
        cls.trunks = []
        cls.network_segment_ranges = []
        cls.conntrack_helpers = []
        cls.ndp_proxies = []

    @classmethod
    def reserve_external_subnet_cidrs(cls):
        client = cls.os_admin.network_client
        ext_nets = client.list_networks(
            **{"router:external": True})['networks']
        for ext_net in ext_nets:
            ext_subnets = client.list_subnets(
                network_id=ext_net['id'])['subnets']
            for ext_subnet in ext_subnets:
                cls.reserve_subnet_cidr(ext_subnet['cidr'])

    @classmethod
    def resource_cleanup(cls):
        if CONF.service_available.neutron:
            # Clean up trunks
            for trunk in cls.trunks:
                cls._try_delete_resource(cls.delete_trunk, trunk)

            # Clean up ndp proxy
            for ndp_proxy in cls.ndp_proxies:
                cls._try_delete_resource(cls.delete_ndp_proxy, ndp_proxy)

            # Clean up port forwardings
            for pf in cls.port_forwardings:
                cls._try_delete_resource(cls.delete_port_forwarding, pf)

            # Clean up floating IPs
            for floating_ip in cls.floating_ips:
                cls._try_delete_resource(cls.delete_floatingip, floating_ip)

            # Clean up Local IP Associations
            for association in cls.local_ip_associations:
                cls._try_delete_resource(cls.delete_local_ip_association,
                                         association)
            # Clean up Local IPs
            for local_ip in cls.local_ips:
                cls._try_delete_resource(cls.delete_local_ip,
                                         local_ip)

            # Clean up conntrack helpers
            for cth in cls.conntrack_helpers:
                cls._try_delete_resource(cls.delete_conntrack_helper, cth)

            # Clean up routers
            for router in cls.routers:
                cls._try_delete_resource(cls.delete_router,
                                         router)
            # Clean up metering label rules
            for metering_label_rule in cls.metering_label_rules:
                cls._try_delete_resource(
                    cls.admin_client.delete_metering_label_rule,
                    metering_label_rule['id'])
            # Clean up metering labels
            for metering_label in cls.metering_labels:
                cls._try_delete_resource(
                    cls.admin_client.delete_metering_label,
                    metering_label['id'])
            # Clean up flavors
            for flavor in cls.flavors:
                cls._try_delete_resource(
                    cls.admin_client.delete_flavor,
                    flavor['id'])
            # Clean up service profiles
            for service_profile in cls.service_profiles:
                cls._try_delete_resource(
                    cls.admin_client.delete_service_profile,
                    service_profile['id'])
            # Clean up ports
            for port in cls.ports:
                cls._try_delete_resource(cls.client.delete_port,
                                         port['id'])
            # Clean up subnets
            for subnet in cls.subnets:
                cls._try_delete_resource(cls.client.delete_subnet,
                                         subnet['id'])
            # Clean up admin subnets
            for subnet in cls.admin_subnets:
                cls._try_delete_resource(cls.admin_client.delete_subnet,
                                         subnet['id'])
            # Clean up networks
            for network in cls.networks:
                cls._try_delete_resource(cls.delete_network, network)

            # Clean up admin networks
            for network in cls.admin_networks:
                cls._try_delete_resource(cls.admin_client.delete_network,
                                         network['id'])

            # Clean up security groups
            for security_group in cls.security_groups:
                cls._try_delete_resource(cls.delete_security_group,
                                         security_group)

            # Clean up admin security groups
            for security_group in cls.admin_security_groups:
                cls._try_delete_resource(cls.delete_security_group,
                                         security_group,
                                         client=cls.admin_client)

            # Clean up security group rule templates
            for sg_rule_template in cls.sg_rule_templates:
                cls._try_delete_resource(
                    cls.admin_client.delete_default_security_group_rule,
                    sg_rule_template['id'])

            for subnetpool in cls.subnetpools:
                cls._try_delete_resource(cls.client.delete_subnetpool,
                                         subnetpool['id'])

            for subnetpool in cls.admin_subnetpools:
                cls._try_delete_resource(cls.admin_client.delete_subnetpool,
                                         subnetpool['id'])

            for address_scope in cls.address_scopes:
                cls._try_delete_resource(cls.client.delete_address_scope,
                                         address_scope['id'])

            for address_scope in cls.admin_address_scopes:
                cls._try_delete_resource(
                    cls.admin_client.delete_address_scope,
                    address_scope['id'])

            for project in cls.projects:
                cls._try_delete_resource(
                    cls.identity_admin_client.delete_project,
                    project['id'])

            # Clean up QoS rules
            for qos_rule in cls.qos_rules:
                cls._try_delete_resource(cls.admin_client.delete_qos_rule,
                                         qos_rule['id'])
            # Clean up QoS policies
            # as all networks and ports are already removed, QoS policies
            # shouldn't be "in use"
            for qos_policy in cls.qos_policies:
                cls._try_delete_resource(cls.admin_client.delete_qos_policy,
                                         qos_policy['id'])

            # Clean up log_objects
            for log_object in cls.log_objects:
                cls._try_delete_resource(cls.admin_client.delete_log,
                                         log_object['id'])

            for keypair in cls.keypairs:
                cls._try_delete_resource(cls.delete_keypair, keypair)

            # Clean up network_segment_ranges
            for network_segment_range in cls.network_segment_ranges:
                cls._try_delete_resource(
                    cls.admin_client.delete_network_segment_range,
                    network_segment_range['id'])

        super(BaseNetworkTest, cls).resource_cleanup()

    @classmethod
    def _try_delete_resource(cls, delete_callable, *args, **kwargs):
        """Cleanup resources in case of test-failure

        Some resources are explicitly deleted by the test.
        If the test failed to delete a resource, this method will execute
        the appropriate delete methods. Otherwise, the method ignores NotFound
        exceptions thrown for resources that were correctly deleted by the
        test.

        :param delete_callable: delete method
        :param args: arguments for delete method
        :param kwargs: keyword arguments for delete method
        """
        try:
            delete_callable(*args, **kwargs)
        # if resource is not found, this means it was deleted in the test
        except lib_exc.NotFound:
            pass

    @classmethod
    def create_network(cls, network_name=None, client=None, external=None,
                       shared=None, provider_network_type=None,
                       provider_physical_network=None,
                       provider_segmentation_id=None, **kwargs):
        """Create a network.

        When client is not provider and admin_client is attribute is not None
        (for example when using BaseAdminNetworkTest base class) and using any
        of the convenience parameters (external, shared, provider_network_type,
        provider_physical_network and provider_segmentation_id) it silently
        uses admin_client. If the network is not shared then it uses the same
        project_id as regular client.

        :param network_name: Human-readable name of the network

        :param client: client to be used for connecting to network service

        :param external: indicates whether the network has an external routing
        facility that's not managed by the networking service.

        :param shared: indicates whether this resource is shared across all
        projects. By default, only administrative users can change this value.
        If True and admin_client attribute is not None, then the network is
        created under administrative project.

        :param provider_network_type: the type of physical network that this
        network should be mapped to. For example, 'flat', 'vlan', 'vxlan', or
        'gre'. Valid values depend on a networking back-end.

        :param provider_physical_network: the physical network where this
        network should be implemented. The Networking API v2.0 does not provide
        a way to list available physical networks. For example, the Open
        vSwitch plug-in configuration file defines a symbolic name that maps to
        specific bridges on each compute host.

        :param provider_segmentation_id: The ID of the isolated segment on the
        physical network. The network_type attribute defines the segmentation
        model. For example, if the network_type value is 'vlan', this ID is a
        vlan identifier. If the network_type value is 'gre', this ID is a gre
        key.

        :param **kwargs: extra parameters to be forwarded to network service
        """

        name = (network_name or kwargs.pop('name', None) or
                data_utils.rand_name('test-network-'))

        # translate convenience parameters
        admin_client_required = False
        if provider_network_type:
            admin_client_required = True
            kwargs['provider:network_type'] = provider_network_type
        if provider_physical_network:
            admin_client_required = True
            kwargs['provider:physical_network'] = provider_physical_network
        if provider_segmentation_id:
            admin_client_required = True
            kwargs['provider:segmentation_id'] = provider_segmentation_id
        if external is not None:
            admin_client_required = True
            kwargs['router:external'] = bool(external)
        if shared is not None:
            admin_client_required = True
            kwargs['shared'] = bool(shared)

        if not client:
            if admin_client_required and cls.admin_client:
                # For convenience silently switch to admin client
                client = cls.admin_client
                if not shared:
                    # Keep this network visible from current project
                    project_id = (kwargs.get('project_id') or
                                  kwargs.get('tenant_id') or
                                  cls.client.project_id)
                    kwargs.update(project_id=project_id, tenant_id=project_id)
            else:
                # Use default client
                client = cls.client

        network = client.create_network(name=name, **kwargs)['network']
        network['client'] = client
        cls.networks.append(network)
        return network

    @classmethod
    def delete_network(cls, network, client=None):
        client = client or network.get('client') or cls.client
        client.delete_network(network['id'])

    @classmethod
    def create_shared_network(cls, network_name=None, **kwargs):
        return cls.create_network(name=network_name, shared=True, **kwargs)

    @classmethod
    def create_subnet(cls, network, gateway='', cidr=None, mask_bits=None,
                      ip_version=None, client=None, reserve_cidr=True,
                      allocation_pool_size=None, **kwargs):
        """Wrapper utility that returns a test subnet.

        Convenient wrapper for client.create_subnet method. It reserves and
        allocates CIDRs to avoid creating overlapping subnets.

        :param network: network where to create the subnet
        network['id'] must contain the ID of the network

        :param gateway: gateway IP address
        It can be a str or a netaddr.IPAddress
        If gateway is not given, then it will use default address for
        given subnet CIDR, like "192.168.0.1" for "192.168.0.0/24" CIDR
        if gateway is given as None then no gateway will be assigned

        :param cidr: CIDR of the subnet to create
        It can be either None, a str or a netaddr.IPNetwork instance

        :param mask_bits: CIDR prefix length
        It can be either None or a numeric value.
        If cidr parameter is given then mask_bits is used to determinate a
        sequence of valid CIDR to use as generated.
        Please see netaddr.IPNetwork.subnet method documentation[1]

        :param ip_version: ip version of generated subnet CIDRs
        It can be None, IP_VERSION_4 or IP_VERSION_6
        It has to match given either given CIDR and gateway

        :param ip_version: numeric value (either IP_VERSION_4 or IP_VERSION_6)
        this value must match CIDR and gateway IP versions if any of them is
        given

        :param client: client to be used to connect to network service

        :param reserve_cidr: if True then it reserves assigned CIDR to avoid
        using the same CIDR for further subnets in the scope of the same
        test case class

        :param allocation_pool_size: if the CIDR is not defined, this method
        will assign one in ``get_subnet_cidrs``. Once done, the allocation pool
        will be defined reserving the number of IP addresses requested,
        starting from the end of the assigned CIDR.

        :param **kwargs: optional parameters to be forwarded to wrapped method

        [1] http://netaddr.readthedocs.io/en/latest/tutorial_01.html#supernets-and-subnets  # noqa
        """
        def allocation_pool(cidr, pool_size):
            start = str(netaddr.IPAddress(cidr.last) - pool_size)
            end = str(netaddr.IPAddress(cidr.last) - 1)
            return {'start': start, 'end': end}

        # allow tests to use admin client
        if not client:
            client = cls.client

        if gateway:
            gateway_ip = netaddr.IPAddress(gateway)
            if ip_version:
                if ip_version != gateway_ip.version:
                    raise ValueError(
                        "Gateway IP version doesn't match IP version")
            else:
                ip_version = gateway_ip.version
        else:
            ip_version = ip_version or cls._ip_version

        for subnet_cidr in cls.get_subnet_cidrs(
                ip_version=ip_version, cidr=cidr, mask_bits=mask_bits):
            if gateway is not None:
                kwargs['gateway_ip'] = str(gateway or (subnet_cidr.ip + 1))
            else:
                kwargs['gateway_ip'] = None
            if allocation_pool_size:
                kwargs['allocation_pools'] = [
                    allocation_pool(subnet_cidr, allocation_pool_size)]
            try:
                body = client.create_subnet(
                    network_id=network['id'],
                    cidr=str(subnet_cidr),
                    ip_version=subnet_cidr.version,
                    **kwargs)
                break
            except lib_exc.BadRequest as e:
                if 'overlaps with another subnet' not in str(e):
                    raise
        else:
            message = 'Available CIDR for subnet creation could not be found'
            raise ValueError(message)
        subnet = body['subnet']
        if client is cls.client:
            cls.subnets.append(subnet)
        else:
            cls.admin_subnets.append(subnet)
        if reserve_cidr:
            cls.reserve_subnet_cidr(subnet_cidr)
        return subnet

    @classmethod
    def reserve_subnet_cidr(cls, addr, **ipnetwork_kwargs):
        """Reserve given subnet CIDR making sure it is not used by create_subnet

        :param addr: the CIDR address to be reserved
        It can be a str or netaddr.IPNetwork instance

        :param **ipnetwork_kwargs: optional netaddr.IPNetwork constructor
        parameters
        """

        if not cls.try_reserve_subnet_cidr(addr, **ipnetwork_kwargs):
            raise ValueError('Subnet CIDR already reserved: {0!r}'.format(
                addr))

    @classmethod
    def try_reserve_subnet_cidr(cls, addr, **ipnetwork_kwargs):
        """Reserve given subnet CIDR if it hasn't been reserved before

        :param addr: the CIDR address to be reserved
        It can be a str or netaddr.IPNetwork instance

        :param **ipnetwork_kwargs: optional netaddr.IPNetwork constructor
        parameters

        :return: True if it wasn't reserved before, False elsewhere.
        """

        subnet_cidr = netaddr.IPNetwork(addr, **ipnetwork_kwargs)
        if subnet_cidr in cls.reserved_subnet_cidrs:
            return False
        else:
            cls.reserved_subnet_cidrs.add(subnet_cidr)
            return True

    @classmethod
    def get_subnet_cidrs(
            cls, cidr=None, mask_bits=None, ip_version=None):
        """Iterate over a sequence of unused subnet CIDR for IP version

        :param cidr: CIDR of the subnet to create
        It can be either None, a str or a netaddr.IPNetwork instance

        :param mask_bits: CIDR prefix length
        It can be either None or a numeric value.
        If cidr parameter is given then mask_bits is used to determinate a
        sequence of valid CIDR to use as generated.
        Please see netaddr.IPNetwork.subnet method documentation[1]

        :param ip_version: ip version of generated subnet CIDRs
        It can be None, IP_VERSION_4 or IP_VERSION_6
        It has to match given CIDR if given

        :return: iterator over reserved CIDRs of type netaddr.IPNetwork

        [1] http://netaddr.readthedocs.io/en/latest/tutorial_01.html#supernets-and-subnets  # noqa
        """

        if cidr:
            # Generate subnet CIDRs starting from given CIDR
            # checking it is of requested IP version
            cidr = netaddr.IPNetwork(cidr, version=ip_version)
        else:
            # Generate subnet CIDRs starting from configured values
            ip_version = ip_version or cls._ip_version
            if ip_version == const.IP_VERSION_4:
                mask_bits = mask_bits or config.safe_get_config_value(
                    'network', 'project_network_mask_bits')
                cidr = netaddr.IPNetwork(config.safe_get_config_value(
                    'network', 'project_network_cidr'))
            elif ip_version == const.IP_VERSION_6:
                mask_bits = config.safe_get_config_value(
                    'network', 'project_network_v6_mask_bits')
                cidr = netaddr.IPNetwork(config.safe_get_config_value(
                    'network', 'project_network_v6_cidr'))
            else:
                raise ValueError('Invalid IP version: {!r}'.format(ip_version))

        if mask_bits:
            subnet_cidrs = cidr.subnet(mask_bits)
        else:
            subnet_cidrs = iter([cidr])

        for subnet_cidr in subnet_cidrs:
            if subnet_cidr not in cls.reserved_subnet_cidrs:
                yield subnet_cidr

    @classmethod
    def create_port(cls, network, **kwargs):
        """Wrapper utility that returns a test port."""
        if CONF.network.port_vnic_type and 'binding:vnic_type' not in kwargs:
            kwargs['binding:vnic_type'] = CONF.network.port_vnic_type
        if CONF.network.port_profile and 'binding:profile' not in kwargs:
            kwargs['binding:profile'] = CONF.network.port_profile
        body = cls.client.create_port(network_id=network['id'],
                                      **kwargs)
        port = body['port']
        cls.ports.append(port)
        return port

    @classmethod
    def update_port(cls, port, **kwargs):
        """Wrapper utility that updates a test port."""
        body = cls.client.update_port(port['id'],
                                      **kwargs)
        return body['port']

    @classmethod
    def _create_router_with_client(
        cls, client, router_name=None, admin_state_up=False,
        external_network_id=None, enable_snat=None, **kwargs
    ):
        ext_gw_info = {}
        if external_network_id:
            ext_gw_info['network_id'] = external_network_id
        if enable_snat is not None:
            ext_gw_info['enable_snat'] = enable_snat
        body = client.create_router(
            router_name, external_gateway_info=ext_gw_info,
            admin_state_up=admin_state_up, **kwargs)
        router = body['router']
        cls.routers.append(router)
        return router

    @classmethod
    def create_router(cls, *args, **kwargs):
        return cls._create_router_with_client(cls.client, *args, **kwargs)

    @classmethod
    def create_admin_router(cls, *args, **kwargs):
        return cls._create_router_with_client(cls.os_admin.network_client,
                                              *args, **kwargs)

    @classmethod
    def create_floatingip(cls, external_network_id=None, port=None,
                          client=None, **kwargs):
        """Creates a floating IP.

        Create a floating IP and schedule it for later deletion.
        If a client is passed, then it is used for deleting the IP too.

        :param external_network_id: network ID where to create
        By default this is 'CONF.network.public_network_id'.

        :param port: port to bind floating IP to
        This is translated to 'port_id=port['id']'
        By default it is None.

        :param client: network client to be used for creating and cleaning up
        the floating IP.

        :param **kwargs: additional creation parameters to be forwarded to
        networking server.
        """

        client = client or cls.client
        external_network_id = (external_network_id or
                               cls.external_network_id)

        if port:
            port_id = kwargs.setdefault('port_id', port['id'])
            if port_id != port['id']:
                message = "Port ID specified twice: {!s} != {!s}".format(
                    port_id, port['id'])
                raise ValueError(message)

        fip = client.create_floatingip(external_network_id,
                                       **kwargs)['floatingip']

        # save client to be used later in cls.delete_floatingip
        # for final cleanup
        fip['client'] = client
        cls.floating_ips.append(fip)
        return fip

    @classmethod
    def delete_floatingip(cls, floating_ip, client=None):
        """Delete floating IP

        :param client: Client to be used
        If client is not given it will use the client used to create
        the floating IP, or cls.client if unknown.
        """

        client = client or floating_ip.get('client') or cls.client
        client.delete_floatingip(floating_ip['id'])

    @classmethod
    def create_port_forwarding(cls, fip_id, internal_port_id,
                               internal_port, external_port,
                               internal_ip_address=None, protocol="tcp",
                               client=None):
        """Creates a port forwarding.

        Create a port forwarding and schedule it for later deletion.
        If a client is passed, then it is used for deleting the PF too.

        :param fip_id: The ID of the floating IP address.

        :param internal_port_id: The ID of the Neutron port associated to
        the floating IP port forwarding.

        :param internal_port: The TCP/UDP/other protocol port number of the
        Neutron port fixed IP address associated to the floating ip
        port forwarding.

        :param external_port: The TCP/UDP/other protocol port number of
        the port forwarding floating IP address.

        :param internal_ip_address: The fixed IPv4 address of the Neutron
        port associated to the floating IP port forwarding.

        :param protocol: The IP protocol used in the floating IP port
        forwarding.

        :param client: network client to be used for creating and cleaning up
        the floating IP port forwarding.
        """

        client = client or cls.client

        pf = client.create_port_forwarding(
            fip_id, internal_port_id, internal_port, external_port,
            internal_ip_address, protocol)['port_forwarding']

        # save ID of floating IP associated with port forwarding for final
        # cleanup
        pf['floatingip_id'] = fip_id

        # save client to be used later in cls.delete_port_forwarding
        # for final cleanup
        pf['client'] = client
        cls.port_forwardings.append(pf)
        return pf

    @classmethod
    def update_port_forwarding(cls, fip_id, pf_id, client=None, **kwargs):
        """Wrapper utility for update_port_forwarding."""
        client = client or cls.client
        return client.update_port_forwarding(fip_id, pf_id, **kwargs)

    @classmethod
    def delete_port_forwarding(cls, pf, client=None):
        """Delete port forwarding

        :param client: Client to be used
        If client is not given it will use the client used to create
        the port forwarding, or cls.client if unknown.
        """

        client = client or pf.get('client') or cls.client
        client.delete_port_forwarding(pf['floatingip_id'], pf['id'])

    def create_local_ip(cls, network_id=None,
                        client=None, **kwargs):
        """Creates a Local IP.

        Create a Local IP and schedule it for later deletion.
        If a client is passed, then it is used for deleting the IP too.

        :param network_id: network ID where to create
        By default this is 'CONF.network.public_network_id'.

        :param client: network client to be used for creating and cleaning up
        the Local IP.

        :param **kwargs: additional creation parameters to be forwarded to
        networking server.
        """

        client = client or cls.client
        network_id = (network_id or
                      cls.external_network_id)

        local_ip = client.create_local_ip(network_id,
                                          **kwargs)['local_ip']

        # save client to be used later in cls.delete_local_ip
        # for final cleanup
        local_ip['client'] = client
        cls.local_ips.append(local_ip)
        return local_ip

    @classmethod
    def delete_local_ip(cls, local_ip, client=None):
        """Delete Local IP

        :param client: Client to be used
        If client is not given it will use the client used to create
        the Local IP, or cls.client if unknown.
        """

        client = client or local_ip.get('client') or cls.client
        client.delete_local_ip(local_ip['id'])

    @classmethod
    def create_local_ip_association(cls, local_ip_id, fixed_port_id,
                                    fixed_ip_address=None, client=None):
        """Creates a Local IP association.

        Create a Local IP Association and schedule it for later deletion.
        If a client is passed, then it is used for deleting the association
        too.

        :param local_ip_id: The ID of the Local IP.

        :param fixed_port_id: The ID of the Neutron port
        to be associated with the Local IP

        :param fixed_ip_address: The fixed IPv4 address of the Neutron
        port to be associated with the Local IP

        :param client: network client to be used for creating and cleaning up
        the Local IP Association.
        """

        client = client or cls.client

        association = client.create_local_ip_association(
            local_ip_id, fixed_port_id,
            fixed_ip_address)['port_association']

        # save ID of Local IP  for final cleanup
        association['local_ip_id'] = local_ip_id

        # save client to be used later in
        # cls.delete_local_ip_association for final cleanup
        association['client'] = client
        cls.local_ip_associations.append(association)
        return association

    @classmethod
    def delete_local_ip_association(cls, association, client=None):

        """Delete Local IP Association

        :param client: Client to be used
        If client is not given it will use the client used to create
        the local IP association, or cls.client if unknown.
        """

        client = client or association.get('client') or cls.client
        client.delete_local_ip_association(association['local_ip_id'],
                                           association['fixed_port_id'])

    @classmethod
    def create_router_interface(cls, router_id, subnet_id):
        """Wrapper utility that returns a router interface."""
        interface = cls.client.add_router_interface_with_subnet_id(
            router_id, subnet_id)
        return interface

    @classmethod
    def add_extra_routes_atomic(cls, *args, **kwargs):
        return cls.client.add_extra_routes_atomic(*args, **kwargs)

    @classmethod
    def remove_extra_routes_atomic(cls, *args, **kwargs):
        return cls.client.remove_extra_routes_atomic(*args, **kwargs)

    @classmethod
    def get_supported_qos_rule_types(cls):
        body = cls.client.list_qos_rule_types()
        return [rule_type['type'] for rule_type in body['rule_types']]

    @classmethod
    def create_qos_policy(cls, name, description=None, shared=False,
                          project_id=None, is_default=False):
        """Wrapper utility that returns a test QoS policy."""
        body = cls.admin_client.create_qos_policy(
            name, description, shared, project_id, is_default)
        qos_policy = body['policy']
        cls.qos_policies.append(qos_policy)
        return qos_policy

    @classmethod
    def create_qos_dscp_marking_rule(cls, policy_id, dscp_mark):
        """Wrapper utility that creates and returns a QoS dscp rule."""
        body = cls.admin_client.create_dscp_marking_rule(
            policy_id, dscp_mark)
        qos_rule = body['dscp_marking_rule']
        cls.qos_rules.append(qos_rule)
        return qos_rule

    @classmethod
    def delete_router(cls, router, client=None):
        client = client or cls.client
        if 'routes' in router:
            client.remove_router_extra_routes(router['id'])
        body = client.list_router_interfaces(router['id'])
        interfaces = [port for port in body['ports']
                      if port['device_owner'] in const.ROUTER_INTERFACE_OWNERS]
        for i in interfaces:
            try:
                client.remove_router_interface_with_subnet_id(
                    router['id'], i['fixed_ips'][0]['subnet_id'])
            except lib_exc.NotFound:
                pass
        client.delete_router(router['id'])

    @classmethod
    def create_address_scope(cls, name, is_admin=False, **kwargs):
        if is_admin:
            body = cls.admin_client.create_address_scope(name=name, **kwargs)
            cls.admin_address_scopes.append(body['address_scope'])
        else:
            body = cls.client.create_address_scope(name=name, **kwargs)
            cls.address_scopes.append(body['address_scope'])
        return body['address_scope']

    @classmethod
    def create_subnetpool(cls, name, is_admin=False, client=None, **kwargs):
        if client is None:
            client = cls.admin_client if is_admin else cls.client

        if is_admin:
            body = client.create_subnetpool(name, **kwargs)
            cls.admin_subnetpools.append(body['subnetpool'])
        else:
            body = client.create_subnetpool(name, **kwargs)
            cls.subnetpools.append(body['subnetpool'])
        return body['subnetpool']

    @classmethod
    def create_address_group(cls, name, is_admin=False, **kwargs):
        if is_admin:
            body = cls.admin_client.create_address_group(name=name, **kwargs)
            cls.admin_address_groups.append(body['address_group'])
        else:
            body = cls.client.create_address_group(name=name, **kwargs)
            cls.address_groups.append(body['address_group'])
        return body['address_group']

    @classmethod
    def create_project(cls, name=None, description=None):
        test_project = name or data_utils.rand_name('test_project_')
        test_description = description or data_utils.rand_name('desc_')
        project = cls.identity_admin_client.create_project(
            name=test_project,
            description=test_description)['project']
        cls.projects.append(project)
        # Create a project will create a default security group.
        sgs_list = cls.admin_client.list_security_groups(
            tenant_id=project['id'])['security_groups']
        for security_group in sgs_list:
            # Make sure delete_security_group method will use
            # the admin client for this group
            security_group['client'] = cls.admin_client
            cls.security_groups.append(security_group)
        return project

    @classmethod
    def create_security_group(cls, name=None, project=None, client=None,
                              **kwargs):
        if project:
            client = client or cls.admin_client
            project_id = kwargs.setdefault('project_id', project['id'])
            tenant_id = kwargs.setdefault('tenant_id', project['id'])
            if project_id != project['id'] or tenant_id != project['id']:
                raise ValueError('Project ID specified multiple times')
        else:
            client = client or cls.client

        name = name or data_utils.rand_name(cls.__name__)
        security_group = client.create_security_group(name=name, **kwargs)[
            'security_group']
        security_group['client'] = client
        cls.security_groups.append(security_group)
        return security_group

    @classmethod
    def delete_security_group(cls, security_group, client=None):
        client = client or security_group.get('client') or cls.client
        client.delete_security_group(security_group['id'])

    @classmethod
    def get_security_group(cls, name='default', client=None):
        client = client or cls.client
        security_groups = client.list_security_groups()['security_groups']
        for security_group in security_groups:
            if security_group['name'] == name:
                return security_group
        raise ValueError("No such security group named {!r}".format(name))

    @classmethod
    def create_security_group_rule(cls, security_group=None, project=None,
                                   client=None, ip_version=None, **kwargs):
        if project:
            client = client or cls.admin_client
            project_id = kwargs.setdefault('project_id', project['id'])
            tenant_id = kwargs.setdefault('tenant_id', project['id'])
            if project_id != project['id'] or tenant_id != project['id']:
                raise ValueError('Project ID specified multiple times')

        if 'security_group_id' not in kwargs:
            security_group = (security_group or
                              cls.get_security_group(client=client))

        if security_group:
            client = client or security_group.get('client')
            security_group_id = kwargs.setdefault('security_group_id',
                                                  security_group['id'])
            if security_group_id != security_group['id']:
                raise ValueError('Security group ID specified multiple times.')

        ip_version = ip_version or cls._ip_version
        default_params = (
            constants.DEFAULT_SECURITY_GROUP_RULE_PARAMS[ip_version])
        if (('remote_address_group_id' in kwargs or
             'remote_group_id' in kwargs) and
                'remote_ip_prefix' in default_params):
            default_params.pop('remote_ip_prefix')
        for key, value in default_params.items():
            kwargs.setdefault(key, value)

        client = client or cls.client
        return client.create_security_group_rule(**kwargs)[
            'security_group_rule']

    @classmethod
    def create_default_security_group_rule(cls, **kwargs):
        body = cls.admin_client.create_default_security_group_rule(**kwargs)
        default_sg_rule = body['default_security_group_rule']
        cls.sg_rule_templates.append(default_sg_rule)
        return default_sg_rule

    @classmethod
    def create_keypair(cls, client=None, name=None, **kwargs):
        client = client or cls.os_primary.keypairs_client
        name = name or data_utils.rand_name('keypair-test')
        keypair = client.create_keypair(name=name, **kwargs)['keypair']

        # save client for later cleanup
        keypair['client'] = client
        cls.keypairs.append(keypair)
        return keypair

    @classmethod
    def delete_keypair(cls, keypair, client=None):
        client = (client or keypair.get('client') or
                  cls.os_primary.keypairs_client)
        client.delete_keypair(keypair_name=keypair['name'])

    @classmethod
    def create_trunk(cls, port=None, subports=None, client=None, **kwargs):
        """Create network trunk

        :param port: dictionary containing parent port ID (port['id'])
        :param client: client to be used for connecting to networking service
        :param **kwargs: extra parameters to be forwarded to network service

        :returns: dictionary containing created trunk details
        """
        client = client or cls.client

        if port:
            kwargs['port_id'] = port['id']

        trunk = client.create_trunk(subports=subports, **kwargs)['trunk']
        # Save client reference for later deletion
        trunk['client'] = client
        cls.trunks.append(trunk)
        return trunk

    @classmethod
    def delete_trunk(cls, trunk, client=None, detach_parent_port=True):
        """Delete network trunk

        :param trunk: dictionary containing trunk ID (trunk['id'])

        :param client: client to be used for connecting to networking service
        """
        client = client or trunk.get('client') or cls.client
        trunk.update(client.show_trunk(trunk['id'])['trunk'])

        if not trunk['admin_state_up']:
            # Cannot touch trunk before admin_state_up is True
            client.update_trunk(trunk['id'], admin_state_up=True)
        if trunk['sub_ports']:
            # Removes trunk ports before deleting it
            cls._try_delete_resource(client.remove_subports, trunk['id'],
                                     trunk['sub_ports'])

        # we have to detach the interface from the server before
        # the trunk can be deleted.
        parent_port = {'id': trunk['port_id']}

        def is_parent_port_detached():
            parent_port.update(client.show_port(parent_port['id'])['port'])
            return not parent_port['device_id']

        if detach_parent_port and not is_parent_port_detached():
            # this could probably happen when trunk is deleted and parent port
            # has been assigned to a VM that is still running. Here we are
            # assuming that device_id points to such VM.
            cls.os_primary.compute.InterfacesClient().delete_interface(
                parent_port['device_id'], parent_port['id'])
            utils.wait_until_true(is_parent_port_detached)

        client.delete_trunk(trunk['id'])

    @classmethod
    def create_conntrack_helper(cls, router_id, helper, protocol, port,
                                client=None):
        """Create a conntrack helper

        Create a conntrack helper and schedule it for later deletion. If a
        client is passed, then it is used for deleteing the CTH too.

        :param router_id: The ID of the Neutron router associated to the
        conntrack helper.

        :param helper: The conntrack helper module alias

        :param protocol: The conntrack helper IP protocol used in the conntrack
        helper.

        :param port: The conntrack helper IP protocol port number for the
        conntrack helper.

        :param client: network client to be used for creating and cleaning up
        the conntrack helper.
        """

        client = client or cls.client

        cth = client.create_conntrack_helper(router_id, helper, protocol,
                                             port)['conntrack_helper']

        # save ID of router associated with conntrack helper for final cleanup
        cth['router_id'] = router_id

        # save client to be used later in cls.delete_conntrack_helper for final
        # cleanup
        cth['client'] = client
        cls.conntrack_helpers.append(cth)
        return cth

    @classmethod
    def delete_conntrack_helper(cls, cth, client=None):
        """Delete conntrack helper

        :param client: Client to be used
        If client is not given it will use the client used to create the
        conntrack helper, or cls.client if unknown.
        """

        client = client or cth.get('client') or cls.client
        client.delete_conntrack_helper(cth['router_id'], cth['id'])

    @classmethod
    def create_ndp_proxy(cls, router_id, port_id, client=None, **kwargs):
        """Creates a ndp proxy.

        Create a ndp proxy and schedule it for later deletion.
        If a client is passed, then it is used for deleting the NDP proxy too.

        :param router_id: router ID where to create the ndp proxy.

        :param port_id: port ID which the ndp proxy associate with

        :param client: network client to be used for creating and cleaning up
        the ndp proxy.

        :param **kwargs: additional creation parameters to be forwarded to
        networking server.
        """
        client = client or cls.client

        data = {'router_id': router_id, 'port_id': port_id}
        if kwargs:
            data.update(kwargs)
        ndp_proxy = client.create_ndp_proxy(**data)['ndp_proxy']

        # save client to be used later in cls.delete_ndp_proxy
        # for final cleanup
        ndp_proxy['client'] = client
        cls.ndp_proxies.append(ndp_proxy)
        return ndp_proxy

    @classmethod
    def delete_ndp_proxy(cls, ndp_proxy, client=None):
        """Delete ndp proxy

        :param client: Client to be used
        If client is not given it will use the client used to create
        the ndp proxy, or cls.client if unknown.
        """
        client = client or ndp_proxy.get('client') or cls.client
        client.delete_ndp_proxy(ndp_proxy['id'])


class BaseAdminNetworkTest(BaseNetworkTest):

    credentials = ['primary', 'admin']

    @classmethod
    def setup_clients(cls):
        super(BaseAdminNetworkTest, cls).setup_clients()
        cls.admin_client = cls.os_admin.network_client
        cls.identity_admin_client = cls.os_admin.projects_client

    @classmethod
    def create_metering_label(cls, name, description):
        """Wrapper utility that returns a test metering label."""
        body = cls.admin_client.create_metering_label(
            description=description,
            name=data_utils.rand_name("metering-label"))
        metering_label = body['metering_label']
        cls.metering_labels.append(metering_label)
        return metering_label

    @classmethod
    def create_metering_label_rule(cls, remote_ip_prefix, direction,
                                   metering_label_id):
        """Wrapper utility that returns a test metering label rule."""
        body = cls.admin_client.create_metering_label_rule(
            remote_ip_prefix=remote_ip_prefix, direction=direction,
            metering_label_id=metering_label_id)
        metering_label_rule = body['metering_label_rule']
        cls.metering_label_rules.append(metering_label_rule)
        return metering_label_rule

    @classmethod
    def create_network_segment_range(cls, name, shared,
                                     project_id, network_type,
                                     physical_network, minimum,
                                     maximum):
        """Wrapper utility that returns a test network segment range."""
        network_segment_range_args = {'name': name,
                                      'shared': shared,
                                      'project_id': project_id,
                                      'network_type': network_type,
                                      'physical_network': physical_network,
                                      'minimum': minimum,
                                      'maximum': maximum}
        body = cls.admin_client.create_network_segment_range(
            **network_segment_range_args)
        network_segment_range = body['network_segment_range']
        cls.network_segment_ranges.append(network_segment_range)
        return network_segment_range

    @classmethod
    def create_flavor(cls, name, description, service_type):
        """Wrapper utility that returns a test flavor."""
        body = cls.admin_client.create_flavor(
            description=description, service_type=service_type,
            name=name)
        flavor = body['flavor']
        cls.flavors.append(flavor)
        return flavor

    @classmethod
    def create_service_profile(cls, description, metainfo, driver):
        """Wrapper utility that returns a test service profile."""
        body = cls.admin_client.create_service_profile(
            driver=driver, metainfo=metainfo, description=description)
        service_profile = body['service_profile']
        cls.service_profiles.append(service_profile)
        return service_profile

    @classmethod
    def create_log(cls, name, description=None,
                   resource_type='security_group', resource_id=None,
                   target_id=None, event='ALL', enabled=True):
        """Wrapper utility that returns a test log object."""
        log_args = {'name': name,
                    'resource_type': resource_type,
                    'resource_id': resource_id,
                    'target_id': target_id,
                    'event': event,
                    'enabled': enabled}
        if description:
            log_args['description'] = description
        body = cls.admin_client.create_log(**log_args)
        log_object = body['log']
        cls.log_objects.append(log_object)
        return log_object

    @classmethod
    def get_unused_ip(cls, net_id, ip_version=None):
        """Get an unused ip address in a allocation pool of net"""
        body = cls.admin_client.list_ports(network_id=net_id)
        ports = body['ports']
        used_ips = []
        for port in ports:
            used_ips.extend(
                [fixed_ip['ip_address'] for fixed_ip in port['fixed_ips']])
        body = cls.admin_client.list_subnets(network_id=net_id)
        subnets = body['subnets']

        for subnet in subnets:
            if ip_version and subnet['ip_version'] != ip_version:
                continue
            cidr = subnet['cidr']
            allocation_pools = subnet['allocation_pools']
            iterators = []
            if allocation_pools:
                for allocation_pool in allocation_pools:
                    iterators.append(netaddr.iter_iprange(
                        allocation_pool['start'], allocation_pool['end']))
            else:
                net = netaddr.IPNetwork(cidr)

                def _iterip():
                    for ip in net:
                        if ip not in (net.network, net.broadcast):
                            yield ip
                iterators.append(iter(_iterip()))

            for iterator in iterators:
                for ip in iterator:
                    if str(ip) not in used_ips:
                        return str(ip)

        message = (
            "net(%s) has no usable IP address in allocation pools" % net_id)
        raise exceptions.InvalidConfiguration(message)

    @classmethod
    def create_provider_network(cls, physnet_name, start_segmentation_id,
                                max_attempts=30):
        segmentation_id = start_segmentation_id
        for attempts in range(max_attempts):
            try:
                return cls.create_network(
                    name=data_utils.rand_name('test_net'),
                    shared=True,
                    provider_network_type='vlan',
                    provider_physical_network=physnet_name,
                    provider_segmentation_id=segmentation_id)
            except lib_exc.Conflict:
                segmentation_id += 1
                if segmentation_id > 4095:
                    raise lib_exc.TempestException(
                        "No free segmentation id was found for provider "
                        "network creation!")
                time.sleep(CONF.network.build_interval)
        LOG.exception("Failed to create provider network after "
                      "%d attempts", max_attempts)
        raise lib_exc.TimeoutException


def require_qos_rule_type(rule_type):
    def decorator(f):
        @functools.wraps(f)
        def wrapper(self, *func_args, **func_kwargs):
            if rule_type not in self.get_supported_qos_rule_types():
                raise self.skipException(
                    "%s rule type is required." % rule_type)
            return f(self, *func_args, **func_kwargs)
        return wrapper
    return decorator


def _require_sorting(f):
    @functools.wraps(f)
    def inner(self, *args, **kwargs):
        if not tutils.is_extension_enabled("sorting", "network"):
            self.skipTest('Sorting feature is required')
        return f(self, *args, **kwargs)
    return inner


def _require_pagination(f):
    @functools.wraps(f)
    def inner(self, *args, **kwargs):
        if not tutils.is_extension_enabled("pagination", "network"):
            self.skipTest('Pagination feature is required')
        return f(self, *args, **kwargs)
    return inner


class BaseSearchCriteriaTest(BaseNetworkTest):

    # This should be defined by subclasses to reflect resource name to test
    resource = None

    field = 'name'

    # NOTE(ihrachys): some names, like those starting with an underscore (_)
    # are sorted differently depending on whether the plugin implements native
    # sorting support, or not. So we avoid any such cases here, sticking to
    # alphanumeric. Also test a case when there are multiple resources with the
    # same name
    resource_names = ('test1', 'abc1', 'test10', '123test') + ('test1',)

    force_tenant_isolation = True

    list_kwargs = {}

    list_as_admin = False

    def assertSameOrder(self, original, actual):
        # gracefully handle iterators passed
        original = list(original)
        actual = list(actual)
        self.assertEqual(len(original), len(actual))
        for expected, res in zip(original, actual):
            self.assertEqual(expected[self.field], res[self.field])

    @utils.classproperty
    def plural_name(self):
        return '%ss' % self.resource

    @property
    def list_client(self):
        return self.admin_client if self.list_as_admin else self.client

    def list_method(self, *args, **kwargs):
        method = getattr(self.list_client, 'list_%s' % self.plural_name)
        kwargs.update(self.list_kwargs)
        return method(*args, **kwargs)

    def get_bare_url(self, url):
        base_url = self.client.base_url
        base_url_normalized = utils.normalize_url(base_url)
        url_normalized = utils.normalize_url(url)
        self.assertTrue(url_normalized.startswith(base_url_normalized))
        return url_normalized[len(base_url_normalized):]

    @classmethod
    def _extract_resources(cls, body):
        return body[cls.plural_name]

    def _test_list_sorts(self, direction):
        sort_args = {
            'sort_dir': direction,
            'sort_key': self.field
        }
        body = self.list_method(**sort_args)
        resources = self._extract_resources(body)
        self.assertNotEmpty(
            resources, "%s list returned is empty" % self.resource)
        retrieved_names = [res[self.field] for res in resources]
        # sort without taking into account whether the network is named with
        # a capital letter or not
        expected = sorted(retrieved_names, key=lambda v: v.upper())
        if direction == constants.SORT_DIRECTION_DESC:
            expected = list(reversed(expected))
        self.assertEqual(expected, retrieved_names)

    @_require_sorting
    def _test_list_sorts_asc(self):
        self._test_list_sorts(constants.SORT_DIRECTION_ASC)

    @_require_sorting
    def _test_list_sorts_desc(self):
        self._test_list_sorts(constants.SORT_DIRECTION_DESC)

    @_require_pagination
    def _test_list_pagination(self):
        for limit in range(1, len(self.resource_names) + 1):
            pagination_args = {
                'limit': limit,
            }
            body = self.list_method(**pagination_args)
            resources = self._extract_resources(body)
            self.assertEqual(limit, len(resources))

    @_require_pagination
    def _test_list_no_pagination_limit_0(self):
        pagination_args = {
            'limit': 0,
        }
        body = self.list_method(**pagination_args)
        resources = self._extract_resources(body)
        self.assertGreaterEqual(len(resources), len(self.resource_names))

    def _test_list_pagination_iteratively(self, lister):
        # first, collect all resources for later comparison
        sort_args = {
            'sort_dir': constants.SORT_DIRECTION_ASC,
            'sort_key': self.field
        }
        body = self.list_method(**sort_args)
        expected_resources = self._extract_resources(body)
        self.assertNotEmpty(expected_resources)

        resources = lister(
            len(expected_resources), sort_args
        )

        # finally, compare that the list retrieved in one go is identical to
        # the one containing pagination results
        self.assertSameOrder(expected_resources, resources)

    def _list_all_with_marker(self, niterations, sort_args):
        # paginate resources one by one, using last fetched resource as a
        # marker
        resources = []
        for i in range(niterations):
            pagination_args = sort_args.copy()
            pagination_args['limit'] = 1
            if resources:
                pagination_args['marker'] = resources[-1]['id']
            body = self.list_method(**pagination_args)
            resources_ = self._extract_resources(body)
            self.assertEqual(1, len(resources_))
            resources.extend(resources_)
        return resources

    @_require_pagination
    @_require_sorting
    def _test_list_pagination_with_marker(self):
        self._test_list_pagination_iteratively(self._list_all_with_marker)

    def _list_all_with_hrefs(self, niterations, sort_args):
        # paginate resources one by one, using next href links
        resources = []
        prev_links = {}

        for i in range(niterations):
            if prev_links:
                uri = self.get_bare_url(prev_links['next'])
            else:
                sort_args.update(self.list_kwargs)
                uri = self.list_client.build_uri(
                    self.plural_name, limit=1, **sort_args)
            prev_links, body = self.list_client.get_uri_with_links(
                self.plural_name, uri
            )
            resources_ = self._extract_resources(body)
            self.assertEqual(1, len(resources_))
            resources.extend(resources_)

        # The last element is empty and does not contain 'next' link
        uri = self.get_bare_url(prev_links['next'])
        prev_links, body = self.client.get_uri_with_links(
            self.plural_name, uri
        )
        self.assertNotIn('next', prev_links)

        # Now walk backwards and compare results
        resources2 = []
        for i in range(niterations):
            uri = self.get_bare_url(prev_links['previous'])
            prev_links, body = self.list_client.get_uri_with_links(
                self.plural_name, uri
            )
            resources_ = self._extract_resources(body)
            self.assertEqual(1, len(resources_))
            resources2.extend(resources_)

        self.assertSameOrder(resources, reversed(resources2))

        return resources

    @_require_pagination
    @_require_sorting
    def _test_list_pagination_with_href_links(self):
        self._test_list_pagination_iteratively(self._list_all_with_hrefs)

    @_require_pagination
    @_require_sorting
    def _test_list_pagination_page_reverse_with_href_links(
            self, direction=constants.SORT_DIRECTION_ASC):
        pagination_args = {
            'sort_dir': direction,
            'sort_key': self.field,
        }
        body = self.list_method(**pagination_args)
        expected_resources = self._extract_resources(body)

        page_size = 2
        pagination_args['limit'] = page_size

        prev_links = {}
        resources = []
        num_resources = len(expected_resources)
        niterations = int(math.ceil(float(num_resources) / page_size))
        for i in range(niterations):
            if prev_links:
                uri = self.get_bare_url(prev_links['previous'])
            else:
                pagination_args.update(self.list_kwargs)
                uri = self.list_client.build_uri(
                    self.plural_name, page_reverse=True, **pagination_args)
            prev_links, body = self.list_client.get_uri_with_links(
                self.plural_name, uri
            )
            resources_ = self._extract_resources(body)
            self.assertGreaterEqual(page_size, len(resources_))
            resources.extend(reversed(resources_))

        self.assertSameOrder(expected_resources, reversed(resources))

    @_require_pagination
    @_require_sorting
    def _test_list_pagination_page_reverse_asc(self):
        self._test_list_pagination_page_reverse(
            direction=constants.SORT_DIRECTION_ASC)

    @_require_pagination
    @_require_sorting
    def _test_list_pagination_page_reverse_desc(self):
        self._test_list_pagination_page_reverse(
            direction=constants.SORT_DIRECTION_DESC)

    def _test_list_pagination_page_reverse(self, direction):
        pagination_args = {
            'sort_dir': direction,
            'sort_key': self.field,
            'limit': 3,
        }
        body = self.list_method(**pagination_args)
        expected_resources = self._extract_resources(body)

        pagination_args['limit'] -= 1
        pagination_args['marker'] = expected_resources[-1]['id']
        pagination_args['page_reverse'] = True
        body = self.list_method(**pagination_args)

        self.assertSameOrder(
            # the last entry is not included in 2nd result when used as a
            # marker
            expected_resources[:-1],
            self._extract_resources(body))

    @tutils.requires_ext(extension="filter-validation", service="network")
    def _test_list_validation_filters(
            self, validation_args, filter_is_valid=True):
        if not filter_is_valid:
            self.assertRaises(lib_exc.BadRequest, self.list_method,
                              **validation_args)
        else:
            body = self.list_method(**validation_args)
            resources = self._extract_resources(body)
            for resource in resources:
                self.assertIn(resource['name'], self.resource_names)



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_utils import uuidutils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from neutron_tempest_plugin.api import test_network_ip_availability as net_ip


class NetworksIpAvailabilityNegativeTest(net_ip.NetworksIpAvailabilityTest):

    @decorators.attr(type='negative')
    @decorators.idempotent_id('3b8693eb-6c57-4ea1-ab84-3730c9ee9c84')
    def test_network_availability_nonexistent_network_id(self):
        self.assertRaises(lib_exc.NotFound,
                          self.admin_client.show_network_ip_availability,
                          uuidutils.generate_uuid())



# Copyright 2017 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from tempest.common import utils
from tempest.common import waiters
from tempest.lib import decorators

from neutron_tempest_plugin.common import ssh
from neutron_tempest_plugin import config
from neutron_tempest_plugin.scenario import base
from neutron_tempest_plugin.scenario import constants as const

CONF = config.CONF


class FloatingIpTestCasesAdmin(base.BaseTempestTestCase):
    credentials = ['primary', 'admin']

    @classmethod
    def setup_clients(cls):
        super(FloatingIpTestCasesAdmin, cls).setup_clients()
        # admin_client set in BaseAdminNetworkTest but here we inherit from
        # BaseNetworkTest
        if not cls.admin_client:
            cls.admin_client = cls.os_admin.network_client

    @classmethod
    @utils.requires_ext(extension="router", service="network")
    def resource_setup(cls):
        super(FloatingIpTestCasesAdmin, cls).resource_setup()
        cls.network = cls.create_network()
        cls.create_subnet(cls.network)
        router = cls.create_router_by_client()
        cls.create_router_interface(router['id'], cls.subnets[0]['id'])
        # Create keypair with admin privileges
        cls.keypair = cls.create_keypair(client=cls.os_admin.keypairs_client)

        # Create security group with admin privileges
        network_client = cls.os_admin.network_client
        cls.secgroup = cls.create_security_group(
            client=cls.os_admin.network_client)
        cls.create_loginable_secgroup_rule(
            secgroup_id=cls.secgroup['id'],
            client=network_client)
        cls.create_pingable_secgroup_rule(
            secgroup_id=cls.secgroup['id'],
            client=network_client),

    def _list_hypervisors(self):
        # List of hypervisors
        return self.os_admin.hv_client.list_hypervisors()['hypervisors']

    def _list_availability_zones(self):
        # List of availability zones
        func = self.os_admin.az_client.list_availability_zones
        return func()['availabilityZoneInfo']

    def _create_vms(self, hyper, avail_zone, num_servers=2):
        servers, fips, server_ssh_clients = ([], [], [])
        # Create the availability zone with default zone and
        # a specific mentioned hypervisor.
        az = avail_zone + ':' + hyper
        for i in range(num_servers):
            servers.append(self.create_server(
                flavor_ref=CONF.compute.flavor_ref,
                image_ref=CONF.compute.image_ref,
                key_name=self.keypair['name'],
                networks=[{'uuid': self.network['id']}],
                security_groups=[{'name': self.secgroup['name']}],
                availability_zone=az))
        for i, server in enumerate(servers):
            waiters.wait_for_server_status(
                self.os_admin.servers_client, server['server']['id'],
                const.SERVER_STATUS_ACTIVE)
            port = self.admin_client.list_ports(
                network_id=self.network['id'],
                device_id=server['server']['id']
            )['ports'][0]
            fip = self.create_floatingip(port=port,
                                         client=self.os_admin.network_client)
            fips.append(fip)
            server_ssh_clients.append(ssh.Client(
                fips[i]['floating_ip_address'], CONF.validation.image_ssh_user,
                pkey=self.keypair['private_key']))
        return servers, server_ssh_clients, fips

    @decorators.idempotent_id('6bba729b-3fb6-494b-9e1e-82bbd89a1045')
    def test_two_vms_fips(self):
        """Test two VMs floating IPs

        This test verifies the ability of two instances
        that were created in the same compute node and same availability zone
        to reach each other.
        """
        # Get hypervisor list to pass it for vm creation
        hyper = self._list_hypervisors()[0]['hypervisor_hostname']
        # Get availability zone list to pass it for vm creation
        avail_zone = self._list_availability_zones()[0]['zoneName']
        servers, server_ssh_clients, fips = self._create_vms(hyper, avail_zone)
        self.check_remote_connectivity(
            server_ssh_clients[0], fips[1]['floating_ip_address'],
            servers=servers)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib import decorators
import testtools

from neutron_tempest_plugin.api import base
from neutron_tempest_plugin import config


class NetworksTestJSON(base.BaseNetworkTest):
    """Test Networks

    Tests the following operations in the Neutron API using the REST client for
    Neutron:

        list tenant's networks
        show a network
        show a tenant network details

    v2.0 of the Neutron API is assumed.
    """

    @classmethod
    def resource_setup(cls):
        super(NetworksTestJSON, cls).resource_setup()
        cls.network = cls.create_network()

    @decorators.idempotent_id('2bf13842-c93f-4a69-83ed-717d2ec3b44e')
    def test_show_network(self):
        # Verify the details of a network
        body = self.client.show_network(self.network['id'])
        network = body['network']
        fields = ['id', 'name']
        if utils.is_extension_enabled('net-mtu', 'network'):
            fields.append('mtu')
        for key in fields:
            self.assertEqual(network[key], self.network[key])
        project_id = self.client.project_id
        self.assertEqual(project_id, network['tenant_id'])
        if utils.is_extension_enabled('project-id', 'network'):
            self.assertEqual(project_id, network['project_id'])

    @decorators.idempotent_id('26f2b7a5-2cd1-4f3a-b11f-ad259b099b11')
    @utils.requires_ext(extension="project-id", service="network")
    def test_show_network_fields_keystone_v3(self):

        def _check_show_network_fields(fields, expect_project_id,
                                       expect_tenant_id):
            params = {}
            if fields:
                params['fields'] = fields
            body = self.client.show_network(self.network['id'], **params)
            network = body['network']
            self.assertEqual(expect_project_id, 'project_id' in network)
            self.assertEqual(expect_tenant_id, 'tenant_id' in network)

        _check_show_network_fields(None, True, True)
        _check_show_network_fields(['tenant_id'], False, True)
        _check_show_network_fields(['project_id'], True, False)
        _check_show_network_fields(['project_id', 'tenant_id'], True, True)

    @decorators.idempotent_id('0cc0552f-afaf-4231-b7a7-c2a1774616da')
    @utils.requires_ext(extension="project-id", service="network")
    def test_create_network_with_project(self):
        project_id = self.client.project_id

        name = 'created-with-project_id'
        network = self.create_network(name, project_id=project_id)
        self.assertEqual(name, network['name'])
        self.assertEqual(project_id, network['project_id'])
        self.assertEqual(project_id, network['tenant_id'])

        observed_network = self.client.list_networks(
            id=network['id'])['networks'][0]
        self.assertEqual(name, observed_network['name'])

        new_name = 'create-with-project_id-2'
        updated_network = self.client.update_network(
            network['id'], name=new_name)['network']
        self.assertEqual(new_name, updated_network['name'])
        self.assertEqual(project_id, updated_network['project_id'])
        self.assertEqual(project_id, updated_network['tenant_id'])

    @decorators.idempotent_id('94e2a44c-3367-4253-8c2a-22deaf59e96c')
    @utils.requires_ext(extension="dns-integration",
                        service="network")
    def test_create_update_network_dns_domain(self):
        domain1 = 'test.org.'
        body = self.create_network(dns_domain=domain1)
        self.assertEqual(domain1, body['dns_domain'])
        net_id = body['id']
        body = self.client.list_networks(id=net_id)['networks'][0]
        self.assertEqual(domain1, body['dns_domain'])
        domain2 = 'd.org.'
        body = self.client.update_network(net_id, dns_domain=domain2)
        self.assertEqual(domain2, body['network']['dns_domain'])
        body = self.client.show_network(net_id)['network']
        self.assertEqual(domain2, body['dns_domain'])

    @decorators.idempotent_id('a23186b9-aa6f-4b08-b877-35ca3b9cd54c')
    @utils.requires_ext(extension="project-id", service="network")
    def test_list_networks_fields_keystone_v3(self):
        def _check_list_networks_fields(fields, expect_project_id,
                                        expect_tenant_id):
            params = {}
            if fields:
                params['fields'] = fields
            body = self.client.list_networks(**params)
            networks = body['networks']
            self.assertNotEmpty(networks, "Network list returned is empty")
            for network in networks:
                self.assertEqual(expect_project_id, 'project_id' in network)
                self.assertEqual(expect_tenant_id, 'tenant_id' in network)

        _check_list_networks_fields(None, True, True)
        _check_list_networks_fields(['tenant_id'], False, True)
        _check_list_networks_fields(['project_id'], True, False)
        _check_list_networks_fields(['project_id', 'tenant_id'], True, True)


# TODO(ihrachys): check that bad mtu is not allowed; current API extension
# definition doesn't enforce values
# TODO(ihrachys): check that new segment reservation updates mtu, once
# https://review.opendev.org/#/c/353115/ is merged
class NetworksMtuTestJSON(base.BaseNetworkTest):
    required_extensions = ['net-mtu', 'net-mtu-writable']

    @decorators.idempotent_id('c79dbf94-ee26-420f-a56f-382aaccb1a41')
    def test_create_network_custom_mtu(self):
        # 68 should be supported by all implementations, as per api-ref
        network = self.create_network(mtu=68)
        body = self.client.show_network(network['id'])['network']
        self.assertEqual(68, body['mtu'])

    @decorators.idempotent_id('2d35d49d-9d16-465c-92c7-4768eb717688')
    @testtools.skipUnless(config.CONF.network_feature_enabled.ipv6,
                          'IPv6 is not enabled')
    def test_update_network_custom_mtu(self):
        # 68 should be supported by all implementations, as per api-ref
        network = self.create_network(mtu=68)
        body = self.client.show_network(network['id'])['network']
        self.assertEqual(68, body['mtu'])

        # 1280 should be supported by all ipv6 compliant implementations
        self.client.update_network(network['id'], mtu=1280)
        body = self.client.show_network(network['id'])['network']
        self.assertEqual(1280, body['mtu'])


class NetworksSearchCriteriaTest(base.BaseSearchCriteriaTest):

    resource = 'network'

    list_kwargs = {'shared': False, 'router:external': False}

    @classmethod
    def resource_setup(cls):
        super(NetworksSearchCriteriaTest, cls).resource_setup()
        for name in cls.resource_names:
            cls.create_network(network_name=name)

    @decorators.idempotent_id('de27d34a-bd9d-4516-83d6-81ef723f7d0d')
    def test_list_sorts_asc(self):
        self._test_list_sorts_asc()

    @decorators.idempotent_id('e767a160-59f9-4c4b-8dc1-72124a68640a')
    def test_list_sorts_desc(self):
        self._test_list_sorts_desc()

    @decorators.idempotent_id('71389852-f57b-49f2-b109-77b705e9e8af')
    def test_list_pagination(self):
        self._test_list_pagination()

    @decorators.idempotent_id('b7e153d2-37c3-48d4-8390-ec13498fee3d')
    def test_list_pagination_with_marker(self):
        self._test_list_pagination_with_marker()

    @decorators.idempotent_id('8a9c89df-0ee7-4c0d-8f1d-ec8f27cf362f')
    def test_list_pagination_with_href_links(self):
        self._test_list_pagination_with_href_links()

    @decorators.idempotent_id('79a52810-2156-4ab6-b577-9e46e58d4b58')
    def test_list_pagination_page_reverse_asc(self):
        self._test_list_pagination_page_reverse_asc()

    @decorators.idempotent_id('36a4671f-a542-442f-bc44-a8873ee778d1')
    def test_list_pagination_page_reverse_desc(self):
        self._test_list_pagination_page_reverse_desc()

    @decorators.idempotent_id('13eb066c-aa90-406d-b4c3-39595bf8f910')
    def test_list_pagination_page_reverse_with_href_links(self):
        self._test_list_pagination_page_reverse_with_href_links()

    @decorators.idempotent_id('f1867fc5-e1d6-431f-bc9f-8b882e43a7f9')
    def test_list_no_pagination_limit_0(self):
        self._test_list_no_pagination_limit_0()

    @decorators.idempotent_id('3574ec9b-a8b8-43e3-9c11-98f5875df6a9')
    def test_list_validation_filters(self):
        self._test_list_validation_filters(self.list_kwargs)
        self._test_list_validation_filters({
            'unknown_filter': 'value'}, filter_is_valid=False)



# Copyright (c) 2020 Red Hat, Inc.
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from neutron_tempest_plugin.common import ip
from neutron_tempest_plugin.common import ssh
from neutron_tempest_plugin import config
from neutron_tempest_plugin.scenario import base


LOG = logging.getLogger(__name__)
CONF = config.CONF
MIN_VLAN_ID = 1
MAX_VLAN_ID = 4094


class VlanTransparencyTest(base.BaseTempestTestCase):
    credentials = ['primary', 'admin']
    force_tenant_isolation = False

    required_extensions = ['vlan-transparent', 'allowed-address-pairs']

    @classmethod
    def resource_setup(cls):
        super(VlanTransparencyTest, cls).resource_setup()
        # setup basic topology for servers we can log into
        cls.rand_name = data_utils.rand_name(
            cls.__name__.rsplit('.', 1)[-1])
        cls.network = cls.create_network(name=cls.rand_name,
                                         vlan_transparent=True)
        cls.subnet = cls.create_subnet(network=cls.network,
                                       name=cls.rand_name)
        cls.router = cls.create_router_by_client()
        cls.create_router_interface(cls.router['id'], cls.subnet['id'])
        cls.keypair = cls.create_keypair(name=cls.rand_name)
        cls.vm_ports = []
        cls.security_group = cls.create_security_group(name=cls.rand_name)
        cls.create_loginable_secgroup_rule(cls.security_group['id'])

        if CONF.neutron_plugin_options.default_image_is_advanced:
            cls.flavor_ref = CONF.compute.flavor_ref
            cls.image_ref = CONF.compute.image_ref
        else:
            cls.flavor_ref = \
                CONF.neutron_plugin_options.advanced_image_flavor_ref
            cls.image_ref = CONF.neutron_plugin_options.advanced_image_ref

    @classmethod
    def skip_checks(cls):
        super(VlanTransparencyTest, cls).skip_checks()
        if not (CONF.neutron_plugin_options.advanced_image_ref or
                CONF.neutron_plugin_options.default_image_is_advanced):
            raise cls.skipException(
                'Advanced image is required to run these tests.')

    def _create_port_and_server(self, index,
                                port_security=True,
                                allowed_address_pairs=None):
        server_name = 'server-%s-%d' % (self.rand_name, index)
        port_name = 'port-%s-%d' % (self.rand_name, index)
        if port_security:
            sec_groups = [self.security_group['id']]
        else:
            sec_groups = None
        self.vm_ports.append(
            self.create_port(network=self.network, name=port_name,
                             security_groups=sec_groups,
                             port_security_enabled=port_security,
                             allowed_address_pairs=allowed_address_pairs))
        return self.create_server(flavor_ref=self.flavor_ref,
                                  image_ref=self.image_ref,
                                  key_name=self.keypair['name'],
                                  networks=[{'port': self.vm_ports[-1]['id']}],
                                  name=server_name)['server']

    def _configure_vlan_transparent(self, port, ssh_client,
                                    vlan_tag, vlan_ip):
        ip_command = ip.IPCommand(ssh_client=ssh_client)
        addresses = ip_command.list_addresses(port=port)
        port_iface = ip.get_port_device_name(addresses, port)
        subport_iface = ip_command.configure_vlan_transparent(
            port=port, vlan_tag=vlan_tag, ip_addresses=[vlan_ip])

        for address in ip_command.list_addresses(ip_addresses=vlan_ip):
            self.assertEqual(subport_iface, address.device.name)
            self.assertEqual(port_iface, address.device.parent)
            break
        else:
            self.fail("Sub-port fixed IP not found on server.")

    def _create_ssh_client(self, floating_ip):
        if CONF.neutron_plugin_options.default_image_is_advanced:
            username = CONF.validation.image_ssh_user
        else:
            username = CONF.neutron_plugin_options.advanced_image_ssh_user
        return ssh.Client(host=floating_ip['floating_ip_address'],
                          username=username,
                          pkey=self.keypair['private_key'])

    def _test_basic_vlan_transparency_connectivity(
            self, port_security=True, use_allowed_address_pairs=False):
        vlan_tag = data_utils.rand_int_id(start=MIN_VLAN_ID, end=MAX_VLAN_ID)
        vlan_ipmask_template = '192.168.%d.{ip_last_byte}/24' % (vlan_tag %
                                                                 256)
        vms = []
        vlan_ipmasks = []
        floating_ips = []
        ssh_clients = []

        for i in range(2):
            vlan_ipmasks.append(vlan_ipmask_template.format(
                ip_last_byte=(i + 1) * 10))
            if use_allowed_address_pairs:
                allowed_address_pairs = [{'ip_address': vlan_ipmasks[i]}]
            else:
                allowed_address_pairs = None
            vms.append(self._create_port_and_server(
                index=i,
                port_security=port_security,
                allowed_address_pairs=allowed_address_pairs))
            floating_ips.append(self.create_floatingip(port=self.vm_ports[-1]))
            ssh_clients.append(
                self._create_ssh_client(floating_ip=floating_ips[i]))

            self.check_connectivity(
                host=floating_ips[i]['floating_ip_address'],
                ssh_client=ssh_clients[i])
            self._configure_vlan_transparent(port=self.vm_ports[-1],
                                             ssh_client=ssh_clients[i],
                                             vlan_tag=vlan_tag,
                                             vlan_ip=vlan_ipmasks[i])

        if port_security:
            # Ping from vm0 to vm1 via VLAN interface should fail because
            # we haven't allowed ICMP
            self.check_remote_connectivity(
                ssh_clients[0],
                vlan_ipmasks[1].split('/')[0],
                servers=vms,
                should_succeed=False)

            # allow intra-security-group traffic
            sg_rule = self.create_pingable_secgroup_rule(
                self.security_group['id'])
            self.addCleanup(
                    self.os_primary.network_client.delete_security_group_rule,
                    sg_rule['id'])

        # Ping from vm0 to vm1 via VLAN interface should pass because
        # either port security is disabled or the ICMP sec group rule has been
        # added
        self.check_remote_connectivity(
            ssh_clients[0],
            vlan_ipmasks[1].split('/')[0],
            servers=vms)
        # Ping from vm1 to vm0 and check untagged packets are not dropped
        self.check_remote_connectivity(
            ssh_clients[1],
            self.vm_ports[-2]['fixed_ips'][0]['ip_address'],
            servers=vms)

    @decorators.idempotent_id('a2694e3a-6d4d-4a23-9fcc-c3ed3ef37b16')
    def test_vlan_transparent_port_sec_disabled(self):
        self._test_basic_vlan_transparency_connectivity(
            port_security=False, use_allowed_address_pairs=False)

    @decorators.idempotent_id('2dd03b4f-9c20-4cda-8c6a-40fa453ec69a')
    def test_vlan_transparent_allowed_address_pairs(self):
        self._test_basic_vlan_transparency_connectivity(
            port_security=True, use_allowed_address_pairs=True)



# Copyright (C) 2016 VA Linux Systems Japan K.K.
# Copyright (C) 2016 Fumihiko Kakuma <kakuma at valinux co jp>
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config

from neutron_tempest_plugin.neutron_dynamic_routing.scenario import base

from os_ken.tests.integrated.common import docker_base as ctn_base

CONF = config.CONF


class BgpSpeakerProtoTestBase(base.BgpSpeakerScenarioTestJSONBase):

    def _test_check_neighbor_established(self, ip_version):
        self.bgp_peer_args[0]['peer_ip'] = self.r_as_ip[0].split('/')[0]
        num, subnet = next(self.tnet_gen)
        mask = '/' + str(self.TPool.prefixlen)
        TNet = base.Net(name='', net=subnet, mask=self.TPool.prefixlen,
                        cidr=subnet + mask, router=None)
        TSubNet = base.SubNet(name='', cidr=TNet.cidr, mask=TNet.mask)
        MyRouter = base.Router(name='my-router' + str(num), gw='')
        ext_net_id = self.create_bgp_network(
            ip_version, self.MyScope,
            self.PNet, self.PPool, self.PSubNet,
            self.TPool, [[TNet, TSubNet, MyRouter]])
        speaker_id, peers_ids = self.create_and_add_peers_to_speaker(
            ext_net_id,
            self.bgp_speaker_args,
            [self.bgp_peer_args[0]])
        self.check_remote_as_state(self.dr, self.r_ass[0],
                                   ctn_base.BGP_FSM_ESTABLISHED)

    def _test_check_advertised_tenant_network(self, ip_version):
        self.bgp_peer_args[0]['peer_ip'] = self.r_as_ip[0].split('/')[0]
        num, subnet = next(self.tnet_gen)
        mask = '/' + str(self.TPool.prefixlen)
        TNet = base.Net(name='', net=subnet, mask=self.TPool.prefixlen,
                        cidr=subnet + mask, router=None)
        TSubNet = base.SubNet(name='', cidr=TNet.cidr, mask=TNet.mask)
        MyRouter = base.Router(name='my-router' + str(num), gw='')
        ext_net_id = self.create_bgp_network(
            ip_version, self.MyScope,
            self.PNet, self.PPool, self.PSubNet,
            self.TPool, [[TNet, TSubNet, MyRouter]])
        speaker_id, peers_ids = self.create_and_add_peers_to_speaker(
            ext_net_id,
            self.bgp_speaker_args,
            [self.bgp_peer_args[0]])
        self.check_remote_as_state(self.dr, self.r_ass[0],
                                   ctn_base.BGP_FSM_ESTABLISHED)
        rf = 'ipv' + str(ip_version)
        self.check_remote_as_rib(self.r_ass[0], TNet.cidr, rf,
                                 'nexthop',
                                 self.get_next_hop(speaker_id, TNet.cidr))

    def _test_check_advertised_multiple_tenant_network(self, ip_version):
        self.bgp_peer_args[0]['peer_ip'] = self.r_as_ip[0].split('/')[0]
        tnets = []
        tnets_cidr = []
        for i in range(0, 3):
            num, subnet = next(self.tnet_gen)
            mask = '/' + str(self.TPool.prefixlen)
            TNet = base.Net(name='', net=subnet, mask=self.TPool.prefixlen,
                            cidr=subnet + mask, router=None)
            TSubNet = base.SubNet(name='', cidr=TNet.cidr, mask=TNet.mask)
            MyRouter = base.Router(name='my-router' + str(num), gw='')
            tnets.append([TNet, TSubNet, MyRouter])
            tnets_cidr.append(TNet.cidr)
        ext_net_id = self.create_bgp_network(
            ip_version, self.MyScope,
            self.PNet, self.PPool, self.PSubNet,
            self.TPool, tnets)
        speaker_id, peers_ids = self.create_and_add_peers_to_speaker(
            ext_net_id,
            self.bgp_speaker_args,
            [self.bgp_peer_args[0]])
        self.check_remote_as_state(self.dr, self.r_ass[0],
                                   ctn_base.BGP_FSM_ESTABLISHED)
        rf = 'ipv' + str(ip_version)
        for cidr in tnets_cidr:
            self.check_remote_as_rib(self.r_ass[0], cidr, rf,
                                     'nexthop',
                                     self.get_next_hop(speaker_id, cidr))

    def _test_check_neighbor_established_with_multiple_peers(
            self, ip_version):
        for (bgp_peer_args, r_as_ip) in zip(self.bgp_peer_args,
                                            self.r_as_ip):
            bgp_peer_args['peer_ip'] = r_as_ip.split('/')[0]
        num, subnet = next(self.tnet_gen)
        mask = '/' + str(self.TPool.prefixlen)
        TNet = base.Net(name='', net=subnet, mask=self.TPool.prefixlen,
                        cidr=subnet + mask, router=None)
        TSubNet = base.SubNet(name='', cidr=TNet.cidr, mask=TNet.mask)
        MyRouter = base.Router(name='my-router' + str(num), gw='')
        ext_net_id = self.create_bgp_network(
            ip_version, self.MyScope,
            self.PNet, self.PPool, self.PSubNet,
            self.TPool, [[TNet, TSubNet, MyRouter]])
        speaker_id, peers_ids = self.create_and_add_peers_to_speaker(
            ext_net_id,
            self.bgp_speaker_args,
            self.bgp_peer_args)
        self.check_multi_remote_as_state(self.dr, self.r_ass,
                                         ctn_base.BGP_FSM_ESTABLISHED)

    def _test_check_advertised_tenant_network_with_multiple_peers(
            self, ip_version):
        for (bgp_peer_args, r_as_ip) in zip(self.bgp_peer_args,
                                            self.r_as_ip):
            bgp_peer_args['peer_ip'] = r_as_ip.split('/')[0]
        num, subnet = next(self.tnet_gen)
        mask = '/' + str(self.TPool.prefixlen)
        TNet = base.Net(name='', net=subnet, mask=self.TPool.prefixlen,
                        cidr=subnet + mask, router=None)
        TSubNet = base.SubNet(name='', cidr=TNet.cidr, mask=TNet.mask)
        MyRouter = base.Router(name='my-router' + str(num), gw='')
        ext_net_id = self.create_bgp_network(
            ip_version, self.MyScope,
            self.PNet, self.PPool, self.PSubNet,
            self.TPool, [[TNet, TSubNet, MyRouter]])
        speaker_id, peers_ids = self.create_and_add_peers_to_speaker(
            ext_net_id,
            self.bgp_speaker_args,
            self.bgp_peer_args)
        self.check_multi_remote_as_state(self.dr, self.r_ass,
                                         ctn_base.BGP_FSM_ESTABLISHED)
        rf = 'ipv' + str(ip_version)
        next_hop = self.get_next_hop(speaker_id, TNet.cidr)
        self.check_multi_remote_as_rib(self.r_ass, TNet.cidr, rf,
                                       'nexthop', next_hop)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api.definitions import qos as qos_apidef
from neutron_lib import constants as n_constants
from neutron_lib.db import constants as db_const
from neutron_lib.services.qos import constants as qos_consts
from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from neutron_tempest_plugin.api import base

LONG_NAME_NG = 'z' * (db_const.NAME_FIELD_SIZE + 1)
LONG_DESCRIPTION_NG = 'z' * (db_const.LONG_DESCRIPTION_FIELD_SIZE + 1)
LONG_TENANT_ID_NG = 'z' * (db_const.PROJECT_ID_FIELD_SIZE + 1)


class QosNegativeTestJSON(base.BaseAdminNetworkTest):

    required_extensions = [qos_apidef.ALIAS]

    @decorators.attr(type='negative')
    @decorators.idempotent_id('b9dce555-d3b3-11e5-950a-54ee757c77da')
    def test_add_policy_with_too_long_name(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.client.create_qos_policy,
                          LONG_NAME_NG, 'test policy desc1', False)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('b9dce444-d3b3-11e5-950a-54ee747c99db')
    def test_add_policy_with_too_long_description(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.client.create_qos_policy,
                          'test-policy', LONG_DESCRIPTION_NG, False)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('b9dce444-d3b3-11e5-950a-54ee757c77dc')
    def test_add_policy_with_too_long_tenant_id(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.client.create_qos_policy,
                          'test-policy', 'test policy desc1',
                          False, LONG_TENANT_ID_NG)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('0e85f3e4-7a93-4187-b847-8f4e835aae1b')
    def test_update_policy_with_too_long_name(self):
        policy = self.create_qos_policy(
            name=data_utils.rand_name('test', 'policy'),
            description='test policy',
            shared=False)
        self.assertRaises(lib_exc.BadRequest,
                          self.client.update_qos_policy, policy['id'],
                          name=LONG_NAME_NG)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('925c7eaf-474b-4a02-a4ba-76a9f82bc45a')
    def test_update_policy_with_too_long_description(self):
        policy = self.create_qos_policy(
            name=data_utils.rand_name('test', 'policy'),
            description='test policy',
            shared=False)
        self.assertRaises(lib_exc.BadRequest,
                          self.client.update_qos_policy, policy['id'],
                          description=LONG_DESCRIPTION_NG)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('88b54ab0-804b-446c-bc19-8e54222d70ef')
    def test_get_non_existent_qos_policy(self):
        non_exist_id = data_utils.rand_name('qos_policy')
        self.assertRaises(lib_exc.NotFound,
                          self.admin_client.show_qos_policy, non_exist_id)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('21050859-1284-4bf5-b05a-13846f83988f')
    def test_update_non_existent_qos_policy(self):
        non_exist_id = data_utils.rand_name('qos_policy')
        self.assertRaises(lib_exc.NotFound,
                          self.admin_client.update_qos_policy, non_exist_id,
                          shared=False)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('09e435b7-44d3-4f9d-8aa8-c295d46b5866')
    def test_delete_non_existent_qos_policy(self):
        non_exist_id = data_utils.rand_name('qos_policy')
        self.assertRaises(lib_exc.NotFound,
                          self.admin_client.delete_qos_policy, non_exist_id)


class QosRuleNegativeBaseTestJSON(base.BaseAdminNetworkTest):

    required_extensions = [qos_apidef.ALIAS]

    def _test_rule_update_rule_nonexistent_policy(self, create_params,
                                                  update_params):
        non_exist_id = data_utils.rand_name('qos_policy')
        policy = self.create_qos_policy(
            name=data_utils.rand_name('test', 'policy'),
            description='test policy',
            shared=False)
        rule = self.rule_create_m(policy['id'], **create_params)
        if "minimum_bandwidth_rule" in rule.keys():
            rule_id = rule['minimum_bandwidth_rule']['id']
        if "minimum_packet_rate_rule" in rule.keys():
            rule_id = rule['minimum_packet_rate_rule']['id']
        if "bandwidth_limit_rule" in rule.keys():
            rule_id = rule['bandwidth_limit_rule']['id']
        if "dscp_mark" in rule.keys():
            rule_id = rule['id']
        self.assertRaises(
            lib_exc.NotFound,
            self.rule_update_m,
            non_exist_id, rule_id, **update_params)

    def _test_rule_create_rule_non_existent_policy(self, create_params):
        non_exist_id = data_utils.rand_name('qos_policy')
        self.assertRaises(
            lib_exc.NotFound,
            self.rule_create_m,
            non_exist_id, **create_params)

    def _test_rule_update_rule_nonexistent_rule(self, update_params):
        non_exist_id = data_utils.rand_name('qos_rule')
        policy = self.create_qos_policy(name='test-policy',
                                        description='test policy',
                                        shared=False)
        self.assertRaises(
            lib_exc.NotFound,
            self.rule_update_m,
            policy['id'], non_exist_id, **update_params)


class QosBandwidthLimitRuleNegativeTestJSON(QosRuleNegativeBaseTestJSON):

    @classmethod
    def setup_clients(cls):
        super(QosBandwidthLimitRuleNegativeTestJSON, cls).setup_clients()
        cls.qos_bw_limit_rule_client = \
            cls.os_admin.qos_limit_bandwidth_rules_client

    @classmethod
    def resource_setup(cls):
        cls.rule_create_m = \
            cls.qos_bw_limit_rule_client.create_limit_bandwidth_rule
        cls.rule_update_m = \
            cls.qos_bw_limit_rule_client.update_limit_bandwidth_rule
        super(QosBandwidthLimitRuleNegativeTestJSON, cls).resource_setup()

    @decorators.attr(type='negative')
    @decorators.idempotent_id('e9ce8042-c828-4cb9-b1f1-85bd35e6553a')
    def test_rule_update_rule_nonexistent_policy(self):
        create_params = {'max_kbps': 1, 'max_burst_kbps': 1}
        update_params = {'max_kbps': 200, 'max_burst_kbps': 1337}
        self._test_rule_update_rule_nonexistent_policy(
            create_params, update_params)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('1b592566-745f-4e15-a439-073afe341244')
    def test_rule_create_rule_non_existent_policy(self):
        create_params = {'max_kbps': 200, 'max_burst_kbps': 300}
        self._test_rule_create_rule_non_existent_policy(create_params)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('a2c72066-0c32-4f28-be7f-78fa721588b6')
    def test_rule_update_rule_nonexistent_rule(self):
        update_params = {'max_kbps': 200, 'max_burst_kbps': 1337}
        self._test_rule_update_rule_nonexistent_rule(update_params)


class QosMinimumBandwidthRuleNegativeTestJSON(QosRuleNegativeBaseTestJSON):

    @classmethod
    def resource_setup(cls):
        cls.rule_create_m = cls.os_admin.qos_minimum_bandwidth_rules_client.\
            create_minimum_bandwidth_rule
        cls.rule_update_m = cls.os_admin.qos_minimum_bandwidth_rules_client.\
            update_minimum_bandwidth_rule
        super(QosMinimumBandwidthRuleNegativeTestJSON, cls).resource_setup()

    @decorators.attr(type='negative')
    @decorators.idempotent_id('08b8455b-4d4f-4119-bad3-9357085c3a80')
    def test_rule_update_rule_nonexistent_policy(self):
        create_params = {'min_kbps': 1}
        update_params = {'min_kbps': 200}
        self._test_rule_update_rule_nonexistent_policy(
            create_params, update_params)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('5a714a4a-bfbc-4cf9-b0c0-13fd185204f7')
    def test_rule_create_rule_non_existent_policy(self):
        create_params = {'min_kbps': 200}
        self._test_rule_create_rule_non_existent_policy(create_params)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('8470cbe0-8ca5-46ab-9c66-7cf69301b121')
    def test_rule_update_rule_nonexistent_rule(self):
        update_params = {'min_kbps': 200}
        self._test_rule_update_rule_nonexistent_rule(update_params)


class QosMinimumPpsRuleNegativeTestJSON(QosRuleNegativeBaseTestJSON):

    @classmethod
    @utils.requires_ext(service='network',
                        extension='port-resource-request-groups')
    def resource_setup(cls):
        cls.rule_create_m = cls.os_admin.qos_minimum_packet_rate_rules_client.\
            create_minimum_packet_rate_rule
        cls.rule_update_m = cls.os_admin.qos_minimum_packet_rate_rules_client.\
            update_minimum_packet_rate_rule
        super(QosMinimumPpsRuleNegativeTestJSON, cls).resource_setup()

    @decorators.attr(type='negative')
    @decorators.idempotent_id('ddd16824-3e10-11ec-928d-5b1ef3fb9f43')
    def test_rule_update_rule_nonexistent_policy(self):
        create_params = {qos_consts.DIRECTION: n_constants.EGRESS_DIRECTION,
                         qos_consts.MIN_KPPS: 1}
        update_params = {qos_consts.MIN_KPPS: 200}
        self._test_rule_update_rule_nonexistent_policy(
            create_params, update_params)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('de4f5540-3e10-11ec-9700-4bf3629b843e')
    def test_rule_create_rule_non_existent_policy(self):
        create_params = {qos_consts.DIRECTION: n_constants.EGRESS_DIRECTION,
                         qos_consts.MIN_KPPS: 200}
        self._test_rule_create_rule_non_existent_policy(create_params)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('deb914ee-3e10-11ec-b3dc-03e52f9269c9')
    def test_rule_update_rule_nonexistent_rule(self):
        update_params = {qos_consts.MIN_KPPS: 200}
        self._test_rule_update_rule_nonexistent_rule(update_params)


class QosDscpRuleNegativeTestJSON(QosRuleNegativeBaseTestJSON):

    @classmethod
    def resource_setup(cls):
        cls.rule_create_m = cls.create_qos_dscp_marking_rule
        cls.rule_update_m = cls.admin_client.update_dscp_marking_rule
        super(QosDscpRuleNegativeTestJSON, cls).resource_setup()

    @decorators.attr(type='negative')
    @decorators.idempotent_id('d47d5fbe-3e98-476f-b2fd-97818175dea5')
    def test_rule_update_rule_nonexistent_policy(self):
        create_params = {'dscp_mark': 26}
        update_params = {'dscp_mark': 16}
        self._test_rule_update_rule_nonexistent_policy(
            create_params, update_params)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('07d17f09-3dc4-4c24-9bb1-49081a153c5a')
    def test_rule_create_rule_non_existent_policy(self):
        create_params = {'dscp_mark': 16}
        self._test_rule_create_rule_non_existent_policy(create_params)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('9c0bd085-5a7a-496f-a984-50dc631a64f2')
    def test_rule_update_rule_nonexistent_rule(self):
        update_params = {'dscp_mark': 16}
        self._test_rule_update_rule_nonexistent_rule(update_params)



# Copyright (c) 2015 Midokura SARL
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib.common import ssh
from tempest.lib import exceptions as lib_exc

from neutron_tempest_plugin.fwaas.common import fwaas_v2_client
from neutron_tempest_plugin.fwaas.scenario import fwaas_v2_manager as manager

CONF = config.CONF


class FWaaSScenarioTestBase(object):
    def check_connectivity(self, ip_address, username=None, private_key=None,
                           should_connect=True,
                           check_icmp=True, check_ssh=True,
                           check_reverse_icmp_ip=None,
                           should_reverse_connect=True):
        if should_connect:
            msg = "Timed out waiting for %s to become reachable" % ip_address
        else:
            msg = "ip address %s is reachable" % ip_address
        if check_icmp:
            ok = self.ping_ip_address(ip_address,
                                      should_succeed=should_connect)
            self.assertTrue(ok, msg=msg)
        if check_ssh:
            connect_timeout = CONF.validation.connect_timeout
            kwargs = {}
            if not should_connect:
                # Use a shorter timeout for negative case
                kwargs['timeout'] = 1
            try:
                client = ssh.Client(ip_address, username, pkey=private_key,
                                    channel_timeout=connect_timeout,
                                    ssh_key_type=CONF.validation.ssh_key_type,
                                    **kwargs)
                client.test_connection_auth()
                self.assertTrue(should_connect, "Unexpectedly reachable")
                if check_reverse_icmp_ip:
                    cmd = 'ping -c1 -w1 %s' % check_reverse_icmp_ip
                    try:
                        client.exec_command(cmd)
                        self.assertTrue(should_reverse_connect,
                                        "Unexpectedly reachable (reverse)")
                    except lib_exc.SSHExecCommandFailed:
                        if should_reverse_connect:
                            raise
            except lib_exc.SSHTimeout:
                if should_connect:
                    raise


class FWaaSScenarioTest_V2(fwaas_v2_client.FWaaSClientMixin,
                        FWaaSScenarioTestBase,
                        manager.NetworkScenarioTest):
    pass
